import os
import unittest
from pathlib import Path
import pytest
from git import Repo

testing_base = f'/tmp/.neurons-testing-base-{os.getpid()}'
pyel = Path(testing_base, 'compiled')
tel = Path(testing_base)


class TestWrite(unittest.TestCase):
    def setUp(self):
        if not pyel.exists():
            pyel.mkdir(parents=True)  # recrusive clean is called after every test
            (pyel / '__init__.py').touch()

        repo = Repo.init(testing_base)

    def tearDown(self):
        def recursive_clean(path):
            for thing in path.iterdir():
                if thing.is_dir():
                    recursive_clean(thing)
                else:
                    thing.unlink()  # will rm the file

            path.rmdir()

        path = Path(testing_base)
        if path.exists():
            recursive_clean(path)

    def test_0_write_py_after_load_none(self):
        from neurondm import Config, Neuron, Phenotype
        #config = Config('test-write', ttl_export_dir=tel, py_export_dir=pyel)
        #Neuron(Phenotype('TEMP:hello'))
        #config.write()
        config2 = Config('test-write-after-same', ttl_export_dir=tel, py_export_dir=pyel)
        Neuron(Phenotype('TEMP:after-other'))
        config2.write_python()
        # first ttl at write python
        # after loading from another repo location?
        # possibly after write python has already been called once?

    def test_1_write_py_after_load_same(self):
        from neurondm import Config, Neuron, Phenotype
        # trickier
        pytest.skip('TODO')

    def test_2_write_py_after_load_other(self):
        from neurondm.lang import Config, Neuron, Phenotype
        config = Config('huang-2017')
        config.load_existing()

        config2 = Config('test-write-after-other', ttl_export_dir=tel, py_export_dir=pyel)
        Neuron(Phenotype('TEMP:after-other'))
        config2.write_python()



#!/usr/bin/env python3
import inspect
from pathlib import Path
from git.repo import Repo
from rdflib import Graph, URIRef
from neurondm import *
from pyontutils.utils import subclasses
from pyontutils.config import devconfig, checkout_ok as ont_checkout_ok
from neurondm.core import NeuronBase  # FIXME temporary until we can rework the config

__all__ = [
    'AND',
    'OR',
    'graphBase',
    'Config',
    'setLocalContext',
    'getLocalContext',
    'setLocalNames',
    'getLocalNames',
    'resetLocalNames',
    'Phenotype',
    'NegPhenotype',
    'LogicalPhenotype',
    'Neuron',
    'NeuronCUT',
    'NeuronEBM',
    'OntId',
    'OntTerm',
    'ilxtr',  # FIXME
    '_NEURON_CLASS',  # FIXME
    '_CUT_CLASS',
    '_EBM_CLASS',
]


# XXX deprecated kept around until other code can be refactored
def config(remote_base=       'https://raw.githubusercontent.com/SciCrunch/NIF-Ontology/',
           local_base=        None,  # devconfig.ontology_local_repo by default
           branch=            devconfig.neurons_branch,
           core_graph_paths= ['ttl/phenotype-core.ttl',
                              'ttl/phenotypes.ttl'],
           core_graph=        None,
           in_graph_paths=    tuple(),
           out_graph_path=    '/tmp/_Neurons.ttl',
           out_imports=      ['ttl/phenotype-core.ttl'],
           out_graph=         None,
           prefixes=          tuple(),
           force_remote=      False,
           checkout_ok=       ont_checkout_ok,
           scigraph=          None,  # defaults to devconfig.scigraph_api
           iri=               None,
           sources=           tuple(),
           source_file=       None,
           use_local_import_paths=True,
           ignore_existing=   True):
    """ Wraps graphBase.configGraphIO to provide a set of sane defaults
        for input ontologies and output files. """
    graphBase.configGraphIO(remote_base=remote_base,
                            local_base=local_base,
                            branch=branch,
                            core_graph_paths=core_graph_paths,
                            core_graph=core_graph,
                            in_graph_paths=in_graph_paths,
                            out_graph_path=out_graph_path,
                            out_imports=out_imports,
                            out_graph=out_graph,
                            prefixes=prefixes,
                            force_remote=force_remote,
                            checkout_ok=checkout_ok,
                            scigraph=scigraph,
                            iri=iri,
                            sources=sources,
                            source_file=source_file,
                            use_local_import_paths=use_local_import_paths,
                            ignore_existing=ignore_existing)

    pred = graphBase._predicates
    return pred  # because the python module system is opinionated :/

#try:
    #pred = config()
#except FileNotFoundError as e:
    #pred = None
    #from pyontutils.utils import TermColors as tc
    #print(e)
    #print(tc.red('WARNING:'),
          #'config() failed to run at import (see the above error). Please',
          #'call pred = config(*args, **kwargs) again in your local file with',
          #'corrected arguments.')

# set the import to this file instead of neurons
graphBase.__import_name__ = __name__

# add a handy ipython line magic for scig to look up terms
try:
    ip = get_ipython()
    python_magic = ip.find_cell_magic('python')
    def scig_func(*vals):
        python_magic('-m pyontutils.scig ' + ' '.join(vals), '')
    ip.register_magic_function(scig_func, 'line', 'scig')
except NameError:
    pass  # not in an IPython environment so can't register magics


if __name__ == '__main__':
    main()



import unittest
from pathlib import Path
#import sys
#import pudb
#sys.breakpointhook = pudb.set_trace

# TestRoundtrip by itself is not sufficient to induce the cross module version
from test.test_neurons import TestRoundtrip

# write the file manually to show the issue is not related to a previous write
# this works with neurondm.lang or neurondm
test_madness_py = '''
#!/usr/bin/env python3.6
from neurondm import *

class NeuronMarkram2015(NeuronEBM):
    owlClass = ilxtr.NeuronMarkram2015
    shortname = 'Markram2015'


config = Config('test-madness',
                file=__file__,
                ttl_export_dir='/home/tom/git/NIF-Ontology/ttl/generated/neurons')
'''

# this by itself will cause an error due to lack of __init__ in /tmp/
# even without running even_when_transient
#madpath = Path('/tmp/test_madness.py')

madpath = Path('/tmp/madness/test_madness.py')
if not madpath.parent.exists():
    madpath.parent.mkdir()
    (madpath.parent / '__init__.py').touch()

with open(madpath, 'wt') as f:
    f.write(test_madness_py)

# one problem would seem to be having two classes with the same name sourced from different files
# so NeuronMarkram2015 in the serialized file is technically different than the one in the generating
# file, so if they someone come into contact with eachother things go boom (I think) this may require a
# metaclass to solve the issue?
class TestDoNothing(unittest.TestCase):
    def test_rewrite_source_module(self):
        from neurondm import Config
        config = Config('test-madness', py_export_dir=madpath.parent)
        config.load_python()   # this is required
        #breakpoint()
        config.write_python()  # BOOM HEADSHOT



#!/usr/bin/env python3

from neurondm import *

__all__ = ['Test',
           'Layers',
           'PaxRatLayers',
           'Regions',
           'PaxRatRegions',
           'Species',
           'BBP',
           'CUT']

Config()  # explicitly load the core graph TODO need a lighter weight way to do this


class Test(LocalNameManager):
    LOOK_AT_THE_CUTE_LITTLE_GUY = Phenotype('NCBITaxon:10116', 'ilxtr:hasInstanceInSpecies')


class Layers(LocalNameManager):
    # TODO there might be a way to set a default predicate here...
    L1 = Phenotype('UBERON:0005390', 'ilxtr:hasLayerLocationPhenotype')
    L2 = Phenotype('UBERON:0005391', 'ilxtr:hasLayerLocationPhenotype')
    L3 = Phenotype('UBERON:0005392', 'ilxtr:hasLayerLocationPhenotype')
    L23 = LogicalPhenotype(OR, L2, L3)
    L4 = Phenotype('UBERON:0005393', 'ilxtr:hasLayerLocationPhenotype')
    L5 = Phenotype('UBERON:0005394', 'ilxtr:hasLayerLocationPhenotype')
    L6 = Phenotype('UBERON:0005395', 'ilxtr:hasLayerLocationPhenotype')
    L56 = LogicalPhenotype(OR, L5, L6)

    SO = Phenotype('UBERON:0005371', 'ilxtr:hasLayerLocationPhenotype')  # WARNING: uberon has precomposed these, which is annoying
    SPy = Phenotype('UBERON:0002313', 'ilxtr:hasLayerLocationPhenotype')  # SP
    SLA = Phenotype('UBERON:0005370', 'ilxtr:hasLayerLocationPhenotype')
    SLM = Phenotype('UBERON:0007640', 'ilxtr:hasLayerLocationPhenotype')
    SLU = Phenotype('UBERON:0007637', 'ilxtr:hasLayerLocationPhenotype')
    SR = Phenotype('UBERON:0005372', 'ilxtr:hasLayerLocationPhenotype')


class PaxRatLayers(LocalNameManager):
    # TODO there might be a way to set a default predicate here...
    L1 = Phenotype('PAXRAT:509', 'ilxtr:hasLayerLocationPhenotype')
    L2 = Phenotype('PAXRAT:512', 'ilxtr:hasLayerLocationPhenotype')
    L3 = Phenotype('PAXRAT:513', 'ilxtr:hasLayerLocationPhenotype')
    L23 = LogicalPhenotype(OR, L2, L3)
    L4 = Phenotype('PAXRAT:515', 'ilxtr:hasLayerLocationPhenotype')
    L5 = Phenotype('PAXRAT:516', 'ilxtr:hasLayerLocationPhenotype')
    L6 = Phenotype('PAXRAT:519', 'ilxtr:hasLayerLocationPhenotype')

    SO = Phenotype('PAXRAT:675', 'ilxtr:hasLayerLocationPhenotype')  # WARNING: uberon has precomposed these, which is annoying
    SPy = Phenotype('PAXRAT:815', 'ilxtr:hasLayerLocationPhenotype')  # SP
    #SLA = Phenotype('', 'ilxtr:hasLayerLocationPhenotype')  # paxrat does not seem to have this, only LM
    SLM = Phenotype('PAXRAT:443', 'ilxtr:hasLayerLocationPhenotype')
    SLU = Phenotype('PAXRAT:901', 'ilxtr:hasLayerLocationPhenotype')
    SR = Phenotype('PAXRAT:822', 'ilxtr:hasLayerLocationPhenotype')


class Regions(LocalNameManager):
    #('brain = Phenotype('UBERON:0000955', 'ilxtr:hasLocationPhenotype')  # hasLocationPhenotype a high level
    brain = Phenotype('UBERON:0000955', 'ilxtr:hasSomaLocatedIn')
    CTX = Phenotype('UBERON:0001950', 'ilxtr:hasSomaLocatedIn')
    CA1 = Phenotype('UBERON:0003881', 'ilxtr:hasSomaLocatedIn')
    CA2 = Phenotype('UBERON:0003882', 'ilxtr:hasSomaLocatedIn')
    CA3 = Phenotype('UBERON:0003883', 'ilxtr:hasSomaLocatedIn')
    S1 = Phenotype('UBERON:0008933', 'ilxtr:hasSomaLocatedIn')
    V1 = Phenotype('UBERON:0002436', 'ilxtr:hasSomaLocatedIn')


class PaxRatRegions(LocalNameManager):
    brain = Phenotype('UBERON:0000955', 'ilxtr:hasSomaLocatedIn')  # paxinos does not go this high
    CTX = Phenotype('UBERON:0001950', 'ilxtr:hasSomaLocatedIn')  # paxinos does not go this high
    CA1 = Phenotype('PAXRAT:322', 'ilxtr:hasSomaLocatedIn')
    CA2 = Phenotype('PAXRAT:323', 'ilxtr:hasSomaLocatedIn')
    CA3 = Phenotype('PAXRAT:324', 'ilxtr:hasSomaLocatedIn')
    S1 = Phenotype('PAXRAT:794', 'ilxtr:hasSomaLocatedIn')


class Species(LocalNameManager):
    Rat = Phenotype('NCBITaxon:10116', 'ilxtr:hasInstanceInSpecies')
    Mouse = Phenotype('NCBITaxon:10090', 'ilxtr:hasInstanceInSpecies')


class BBP(PaxRatLayers, PaxRatRegions, Species):
    # cat old_neuron_example.py | grep -v ^# | grep -o "\w\+\ \=\ Pheno.\+" | sed "s/^/('/" | sed "s/\ \=\ /',/" | sed 's/Phenotype(//' | sed 's/)/)/'
    PC = Phenotype('ilxtr:PyramidalPhenotype', 'ilxtr:hasMorphologicalPhenotype')  #
    BPC = Phenotype('ilxtr:BiopolarPyramidalPhenotype', 'ilxtr:hasMorphologicalPhenotype')  # collision
    HPC = Phenotype('ilxtr:HorizontalPyramidalPhenotype', 'ilxtr:hasMorphologicalPhenotype')
    IPC = Phenotype('ilxtr:InvertedPyramidalPhenotype', 'ilxtr:hasMorphologicalPhenotype')
    NPC = Phenotype('ilxtr:NarrowPyramidalPhenotype', 'ilxtr:hasMorphologicalPhenotype')  # XXX These are apparently tufted.... so are they NarrowTufted or not? help!
    SPC = Phenotype('ilxtr:StarPyramidalPhenotype', 'ilxtr:hasMorphologicalPhenotype')  # collision with stratum pyramidale
    TPC_C = Phenotype('ilxtr:NarrowTuftedPyramidalPhenotype', 'ilxtr:hasMorphologicalPhenotype')  # are this different?  Yes, the Narrow modifieds the tufted... not entirely sure how that is different from when Narrow modifies Pyramidal...
    TPC = Phenotype('ilxtr:TuftedPyramidalPhenotype', 'ilxtr:hasMorphologicalPhenotype')
    STPC = Phenotype('ilxtr:SlenderTuftedPyramidalPhenotype', 'ilxtr:hasMorphologicalPhenotype')
    TTPC = Phenotype('ilxtr:ThickTuftedPyramidalPhenotype', 'ilxtr:hasMorphologicalPhenotype')
    TTPC1 = Phenotype('ilxtr:LateBifurcatingThickTuftedPyramidalPhenotype', 'ilxtr:hasMorphologicalPhenotype')
    TTPC2 = Phenotype('ilxtr:EarlyBifurcatingThickTuftedPyramidalPhenotype', 'ilxtr:hasMorphologicalPhenotype')
    TPC_A = Phenotype('ilxtr:LateBifurcatingTuftedPyramidalPhenotype', 'ilxtr:hasMorphologicalPhenotype') # these are inconsistent with usages like NGC_DA NGC_SA recomend the NGCDA NGCSA versions since the acronym is better than a/b 1/2
    TPC_B = Phenotype('ilxtr:EarlyBifurcatingTuftedPyramidalPhenotype', 'ilxtr:hasMorphologicalPhenotype')
    UTPC = Phenotype('ilxtr:UntuftedPyramidalPhenotype', 'ilxtr:hasMorphologicalPhenotype')
    BTC = Phenotype('ilxtr:BituftedPhenotype', 'ilxtr:hasMorphologicalPhenotype')
    BP = Phenotype('ilxtr:BipolarPhenotype', 'ilxtr:hasMorphologicalPhenotype')  # NOTE disjoint from biopolar pyramidal... also collision
    BS = Phenotype('ilxtr:BistratifiedPhenotype', 'ilxtr:hasMorphologicalPhenotype')
    ChC = Phenotype('ilxtr:ChandelierPhenotype', 'ilxtr:hasMorphologicalPhenotype')  # AA ??
    DBC = Phenotype('ilxtr:DoubleBouquetPhenotype', 'ilxtr:hasMorphologicalPhenotype')
    DAC = Phenotype('ilxtr:DescendingAxonPhenotype', 'ilxtr:hasMorphologicalPhenotype')  # FIXME how to communicate that these are a bit different than actual 'axon' phenotypes? disjoint from P?
    HAC = Phenotype('ilxtr:HorizontalAxonPhenotype', 'ilxtr:hasMorphologicalPhenotype')
    LAC = Phenotype('ilxtr:LargeAxonPhenotype', 'ilxtr:hasMorphologicalPhenotype')
    SAC = Phenotype('ilxtr:SmallAxonPhenotype', 'ilxtr:hasMorphologicalPhenotype')
    MC = Phenotype('ilxtr:MartinottiPhenotype', 'ilxtr:hasMorphologicalPhenotype')
    NGC = Phenotype('ilxtr:NeurogliaformPhenotype', 'ilxtr:hasMorphologicalPhenotype')
    NGCDA = Phenotype('ilxtr:NeurogliaformDenseAxonPhenotype', 'ilxtr:hasMorphologicalPhenotype')  # NGC_DA
    NGCSA = Phenotype('ilxtr:NeurogliaformSparseAxonPhenotype', 'ilxtr:hasMorphologicalPhenotype')  # NGC_SA
    BC = Phenotype('ilxtr:BasketPhenotype', 'ilxtr:hasMorphologicalPhenotype')
    SBC = Phenotype('ilxtr:SmallBasketPhenotype', 'ilxtr:hasMorphologicalPhenotype')
    LBC = Phenotype('ilxtr:LargeBasketPhenotype', 'ilxtr:hasMorphologicalPhenotype')
    NBC = Phenotype('ilxtr:NestBasketPhenotype', 'ilxtr:hasMorphologicalPhenotype')
    SS = Phenotype('ilxtr:SpinyStellatePhenotype', 'ilxtr:hasMorphologicalPhenotype')  # SS is used on the website, SSC is used on the spreadsheet TODO usecase for non-injective naming...
    AC = Phenotype('ilxtr:PetillaSustainedAccomodatingPhenotype', 'ilxtr:hasElectrophysiologicalPhenotype')
    NAC = Phenotype('ilxtr:PetillaSustainedNonAccomodatingPhenotype', 'ilxtr:hasElectrophysiologicalPhenotype')
    STUT = Phenotype('ilxtr:PetillaSustainedStutteringPhenotype', 'ilxtr:hasElectrophysiologicalPhenotype')
    IR = Phenotype('ilxtr:PetillaSustainedIrregularPhenotype', 'ilxtr:hasElectrophysiologicalPhenotype')
    b = Phenotype('ilxtr:PetillaInitialBurstSpikingPhenotype', 'ilxtr:hasElectrophysiologicalPhenotype')
    c = Phenotype('ilxtr:PetillaInitialClassicalSpikingPhenotype', 'ilxtr:hasElectrophysiologicalPhenotype')
    d = Phenotype('ilxtr:PetillaInitialDelayedSpikingPhenotype', 'ilxtr:hasElectrophysiologicalPhenotype')
    FS = Phenotype('ilxtr:FastSpikingPhenotype', 'ilxtr:hasElectrophysiologicalPhenotype')
    RSNP = Phenotype('ilxtr:RegularSpikingNonPyramidalPhenotype', 'ilxtr:hasElectrophysiologicalPhenotype')
    L1D = Phenotype('UBERON:0005390', 'ilxtr:hasDendriteLocatedIn')
    L4D = Phenotype('UBERON:0005393', 'ilxtr:hasDendriteLocatedIn')
    L1P = Phenotype('UBERON:0005390', 'ilxtr:hasProjectionPhenotype')
    L4P = Phenotype('UBERON:0005393', 'ilxtr:hasProjectionPhenotype')
    SLMP = Phenotype('UBERON:0007640', 'ilxtr:hasProjectionPhenotype')
    ThalP = Phenotype('UBERON:0001897', 'ilxtr:hasProjectionPhenotype')
    CB = Phenotype('PR:000004967', 'ilxtr:hasExpressionPhenotype')
    CCK = Phenotype('PR:000005110', 'ilxtr:hasExpressionPhenotype')
    CR = Phenotype('PR:000004968', 'ilxtr:hasExpressionPhenotype')
    NPY = Phenotype('PR:000011387', 'ilxtr:hasExpressionPhenotype')
    PV = Phenotype('PR:000013502', 'ilxtr:hasExpressionPhenotype')
    #PV = Phenotype('PR:000013502_1', 'ilxtr:hasExpressionPhenotype')  # this errors as expected
    #PVP = Phenotype('PR:000013502', 'ilxtr:hasExpressionPhenotype')  # this errors as expected
    SOM = Phenotype('PR:000015665', 'ilxtr:hasExpressionPhenotype')
    VIP = Phenotype('PR:000017299', 'ilxtr:hasExpressionPhenotype')
    GABA = Phenotype('CHEBI:16865', 'ilxtr:hasExpressionPhenotype')
    D1 = Phenotype('PR:000001175', 'ilxtr:hasExpressionPhenotype')
    DA = Phenotype('CHEBI:18243', 'ilxtr:hasExpressionPhenotype')
    Th = Phenotype('ilxtr:ThickPhenotype', 'ilxtr:hasDendriteMorphologicalPhenotype')
    Tu = Phenotype('ilxtr:TuftedPhenotype', 'ilxtr:hasDendriteMorphologicalPhenotype')
    U = NegPhenotype(Tu)
    S = Phenotype('ilxtr:SlenderPhenotype', 'ilxtr:hasDendriteMorphologicalPhenotype')
    EB = Phenotype('ilxtr:EarlyBifurcatingPhenotype', 'ilxtr:hasDendriteMorphologicalPhenotype')
    LB = Phenotype('ilxtr:LateBifurcatingPhenotype', 'ilxtr:hasDendriteMorphologicalPhenotype')
    Sp = Phenotype('ilxtr:SpinyPhenotype', 'ilxtr:hasDendriteMorphologicalPhenotype')
    ASp = NegPhenotype(Sp)
    PPA = Phenotype('ilxtr:PerforantPathwayAssociated', 'ilxtr:hasPhenotype')  # TODO FIXME
    TRI = Phenotype('ilxtr:TrilaminarPhenotype', 'ilxtr:hasMorphologicalPhenotype')
    IVY = Phenotype('ilxtr:IvyPhenotype', 'ilxtr:hasMorphologicalPhenotype')  # check syns on this?
    SCA = Phenotype('GO:1990021', 'ilxtr:hasProjectionPhenotype')  #'ilxtr:hasAssociatedTractPhenotype')  # how to model these... also schaffer collaterals are a MESS in the ontology FIXME
    #STRI = Phenotype('UBERON:0005383', 'ilxtr:hasSomaLocatedIn')  # VS UBERON:0002435 (always comes up)
    STRI = Phenotype('PAXRAT:168', 'ilxtr:hasSomaLocatedIn')  # VS UBERON:0002435 (always comes up)
    MSN = Phenotype('ilxtr:MediumSpinyPhenotype', 'ilxtr:hasMorphologicalPhenotype')
    INT = Phenotype('ilxtr:InterneuronPhenotype', 'ilxtr:hasCircuitRolePhenotype')  # unsatisfactory
    #CER = Phenotype('UBERON:0002037', 'ilxtr:hasSomaLocatedIn')
    CER = Phenotype('PAXRAT:191', 'ilxtr:hasSomaLocatedIn')
    GRAN = Phenotype('ilxtr:GranulePhenotype', 'ilxtr:hasMorphologicalPhenotype')  # vs granular?


class CUT(LocalNameManager):
    Mammalia = Phenotype('NCBITaxon:40674', ilxtr.hasTaxonRank)
    proj = Phenotype(ilxtr.ProjectionPhenotype, ilxtr.hasCircuitRolePhenotype)
    inter = Phenotype(ilxtr.InterneuronPhenotype, ilxtr.hasCircuitRolePhenotype)
    Ach = Phenotype('SAO:185580330', ilxtr.hasExpressionPhenotype)
    Glu = Phenotype('CHEBI:16015', ilxtr.hasExpressionPhenotype)
    Ser = Phenotype('CHEBI:28790', ilxtr.hasExpressionPhenotype)
    TH = Phenotype('PR:000016301', ilxtr.hasExpressionPhenotype)  # NCBIGene:21823
    TRN = Phenotype('UBERON:0001903', ilxtr.hasSomaLocatedIn)
    Thal = Phenotype('UBERON:0001897', ilxtr.hasSomaLocatedIn)
    MRN = Phenotype('UBERON:0007415', ilxtr.hasSomaLocatedIn)



import os
import unittest
from pathlib import Path
from git import Repo

testing_base = f'/tmp/.neurons-testing-base-{os.getpid()}'
pyel = Path(testing_base, 'compiled')
tel = Path(testing_base)


class _TestNeuronsBase(unittest.TestCase):
    def setUp(self):
        if not pyel.exists():
            pyel.mkdir(parents=True)  # recrusive clean is called after every test
            (pyel / '__init__.py').touch()

        repo = Repo.init(testing_base)

    def tearDown(self):
        def recursive_clean(path):
            for thing in path.iterdir():
                if thing.is_dir():
                    recursive_clean(thing)
                else:
                    thing.unlink()  # will rm the file

            path.rmdir()

        path = Path(testing_base)
        if path.exists():
            recursive_clean(path)



import unittest
import rdflib
from neurondm.core import LabelMaker, Phenotype, Neuron

class TestLabelMaker(unittest.TestCase):

    def setUp(self):
        self.lm = LabelMaker()

        def ns(suffix):
            if isinstance(suffix, rdflib.URIRef):
                raise TypeError('you called this twice')
            return self.lm.predicate_namespace[suffix]

        def ms(*suffixes, p=None):
            return Neuron(*[Phenotype(s, p) for s in suffixes])

        self.ns = ns
        self.ms = ms

    def test_hasCircuitRole(self):
        ns = self.ns
        ms = self.ms
        hcr = ns('hasCircuitRolePhenotype')
        inter = ns('InterneuronPhenotype')
        intrin = ns('IntrinsicPhenotype')
        proj = ns('ProjectionPhenotype')
        prin = ns('PrincipalPhenotype')
        s = ns('SensoryPhenotype')
        m = ns('MotorPhenotype')

        sets = (
            ms(intrin, p=hcr),
            ms(inter, p=hcr),
            ms(prin, p=hcr),
            ms(proj, p=hcr),
            ms(s, p=hcr),
            ms(m, p=hcr),
            ms(inter, intrin, p=hcr),
            ms(intrin, inter, p=hcr),
        )
        wat = next(self.lm.hasCircuitRolePhenotype((Phenotype(inter),)))
        assert wat != 'intrinsic'
        for neuron in sets:
            label = self.lm(neuron)
            print(neuron.pes, label)




#!/usr/bin/env python3.6
""" Converts owl or ttl or raw rdflib graph into a pandas DataFrame. Saved in .pickle format.

Usage:
    allen_cell_types [options]

Options:
    -h --help                   Display this help message
    -v --version                Current version of file
    -r --refresh                Update local copy
    -i --input=<path>           Local copy of Allen Brain Atlas meta data [default: /tmp/allen-cell-types.json]
    -o --output=<path>          Output path of picklized pandas DataFrame [default: allen-cell-types]
"""
import re
import json
from pathlib import Path
import rdflib
import requests
from rdflib.namespace import *
from pyontutils.utils import TermColors as tc, relative_path
from pyontutils.core import simpleOnt
from pyontutils.config import devconfig
from pyontutils.namespaces import makePrefixes, ilxtr, definition
from pyontutils.combinators import annotation
from neurondm.lang import *
from pyontutils.closed_namespaces import rdf, rdfs, owl
from docopt import docopt, parse_defaults
from IPython import embed


class NeuronACT(NeuronEBM):
    owlClass = ilxtr.NeuronACT
    shortname = 'AllenCT'


class AllenCellTypes:

    branch = devconfig.neurons_branch

    phenotype_preds = [
        'hasSomaLocatedIn',
        'hasCircuitRolePhenotype',
        'hasNeurotransmitterPhenotype',
        'hasInstanceInSpecies',
        'hasExperimentalPhenotype',
        'hasDendriteLocatedIn',
        'hasNucleicAcidExpressionPhenotype',
        'hasLayerLocationPhenotype',
        'hasExpressionPhenotype',
        'hasSmallMoleculeExpressionPhenotype',
        'hasPresynapticTerminalsIn',
        'hasPhenotype',
        'hasTaxonRank',
        'hasProteinExpressionPhenotype',
        'hasMorphologicalPhenotype',
        'hasDendriteMorphologicalPhenotype',
        'hasAxonLocatedIn',
        'hasDevelopmentalOrigin',
        'hasElectrophysiologicalPhenotype',
        'hasProjectionPhenotype',
        'hasLocationPhenotype'
    ]

    prefixes = {**{'JAX': 'http://jaxmice.jax.org/strain/',
                   'MMRRC': 'http://www.mmrrc.org/catalog/getSDS.jsp?mmrrc_id=',
                   'AllenTL': 'http://api.brain-map.org/api/v2/data/TransgenicLine/'},
                **makePrefixes('definition', 'ilxtr', 'owl')}

    def __init__(self, input, name):
        self.name = name
        self.ns = {k:rdflib.Namespace(v) for k, v in self.prefixes.items()}
        self.neuron_data = input
        self.tag_names = set()
        # self.sample_neuron()

    def avoid_url_conversion(self, string):
        if not string:
            return string
        return re.sub("/| |\(", '_', string).replace(')', '')

    def sample_neuron(self,):
        Neuron(
            Phenotype('ilxtr:apical',
                      'ilxtr:hasPhenotype',
                      label='apical - truncated'),
            Phenotype('JAX:12345',
                      'ilxtr:hasExperimentalPhenotype',
                      label='prefix+stock_number'),
        )
        # embed()
        print(graphBase.ttl())

    def cell_phenotypes(self, cell_line):
        cell_mappings = {
            'hemisphere': 'ilxtr:hasLocationPhenotype',
            # 'name': 'ilxtr:hasPhenotype',
        }
        phenotypes = []
        for name, value in cell_line.items():
            mapping = cell_mappings.get(name)
            if mapping and value:
                if name == 'hemisphere':
                    if value.lower() == 'left':
                        curie = 'UBERON:0002812'
                    elif value.lower() == 'right':
                        curie = 'UBERON:0002813'
                    else:
                        raise ValueError('got stuck with unkown hemisphere ' + value)
                phenotypes.append(
                    Phenotype(
                        curie,
                        mapping,
                        label=value
                    )
                )
        return phenotypes

    # TODO: wrong phenotype
    def structure_phenotypes(self, cell_line):
        struc = cell_line['structure']
        phenotypes = []
        acronym = self.avoid_url_conversion(struc['acronym'])
        curie = 'MBA:' + str(struc['id'])
        if struc:
            phenotypes.append(
                Phenotype(
                    curie,
                    'ilxtr:hasSomaLocatedIn',
                    label=acronym
                ),
            )
        return phenotypes

    def donor_phenotypes(self, cell_line):
        donor_mappings = {
            'sex_full_name': 'ilxtr:hasBiologicalSex'
        }
        phenotypes = []
        for name, value in cell_line['donor'].items():
            mapping = donor_mappings.get(name)
            if mapping and value:
                if name == 'sex_full_name':
                    if value.lower() == 'male':
                        curie = 'PATO:0000383'
                    elif value.lower() == 'female':
                        curie = 'PATO:0000384'
                    else:
                        raise ValueError('unkown sex ' + str(value))
                phenotypes.append(
                    Phenotype(
                        curie,
                        mapping,
                        label='name'
                    ),
                )
        return phenotypes

    # TODO: Figure how to add: description, name and type
    def transgenic_lines_phenotypes(self, cell_line):
        transgenic_mappings = {
        }
        phenotypes = []
        for tl in cell_line['donor']['transgenic_lines']:
            prefix = tl['transgenic_line_source_name']
            suffix = tl['stock_number'] if tl['stock_number'] else str(tl['id'])
            name = self.avoid_url_conversion(tl['name'])
            _type = tl['transgenic_line_type_name']
            if _type == 'driver':
                pred = 'ilxtr:hasDriverExpressionPhenotype'
            elif _type == 'reporter':
                pred = 'ilxtr:hasReporterExpressionPhenotype'
            else:
                pred = 'ilxtr:hasExpressionPhenotype'

            line_names = []
            if prefix and suffix and prefix in ['AIBS', 'MMRRC', 'JAX']:
                if prefix == 'AIBS':
                    prefix = 'AllenTL'
                iri = self.ns[prefix][suffix]
                phenotypes.append(Phenotype(iri, pred))
        return phenotypes

    # TODO: search if description exists
    # TODO: Create mapping for all possible types
    # TODO: Fork negatives to NegPhenotype
    def specimen_tags_phenotypes(self, cell_line):
        pred = 'ilxtr:hasDendriteMorphologicalPhenotype'
        specimen_tag_mappings = {
            'spiny': Phenotype('ilxtr:SpinyPhenotype', pred),
            'aspiny': NegPhenotype('ilxtr:SpinyPhenotype', pred),
            'sparsely spiny': LogicalPhenotype(AND,
                                               Phenotype('ilxtr:SpinyPhenotype', pred),
                                               Phenotype('PATO:0001609', 'ilxtr:hasPhenotypeModifier')),
            'apicalIntact': Phenotype('ilxtr:ApicalDendritePhenotype', 'ilxtr:hasMorphologicalPhenotype'),
            'apicalTruncated': LogicalPhenotype(AND,
                                                Phenotype('ilxtr:ApicalDendritePhenotype', 'ilxtr:hasMorphologicalPhenotype'),
                                                Phenotype('PATO:0000936', 'ilxtr:hasPhenotypeModifier')),
            'apicalNa': NegPhenotype('ilxtr:ApicalDendritePhenotype', 'ilxtr:hasMorphologicalPhenotype'),  # NA means there was no apical dendrite
        }
        phenotypes = []
        for tag in cell_line['specimen_tags']:
            if 'dendrite type' in tag['name']:
                one_two = tag['name'].split(' - ')[1]
                #if ' ' in one_two:
                    #one, two = one_two.split(' ')
                    #name = one + two.capitalize()
                #else:
                name = one_two
            else:
                one, two = tag['name'].split(' - ')
                #if two == 'NA':  # apical - NA
                    #continue
                name = one + two.capitalize()

            self.tag_names.add(tag['name'])
            # if phenotype == '+':
            if name not in specimen_tag_mappings:
                raise ValueError(name)

            phenotypes.append(specimen_tag_mappings[name]
                              if name in specimen_tag_mappings else
                              Phenotype('ilxtr:' + name, pred))
            # elif phenotype == '-': phenotypes.append(NegPhenotype(...))

        return phenotypes

    # TODO: check to see if specimen_id is really the priority
    def cell_soma_locations_phenotypes(self, cell_line):
        cell_soma_mappings = {
        }
        phenotypes = []
        for csl in cell_line['cell_soma_locations']:
            location = csl['id']
            phenotypes.append(
                Phenotype(
                    'ilxtr:' + str(location),
                    'ilxtr:hasSomaLocatedIn',
                )
            )
        return phenotypes

    def add_mouse_lineage(self, cell_line):
        phenotypes = [Phenotype('NCBITaxon:10090', 'ilxtr:hasInstanceInSpecies')]
        return phenotypes

    def build_phenotypes(self, cell_line):
        phenotype_functions = [
            self.cell_phenotypes,
            self.structure_phenotypes,
            self.donor_phenotypes,
            self.transgenic_lines_phenotypes,
            self.specimen_tags_phenotypes,
            self.add_mouse_lineage,
            # self.cell_soma_locations_phenotypes, # deprecated
        ]
        phenotypes = []
        for func in phenotype_functions:
            phenotypes.extend(func(cell_line))
        return phenotypes

    def build_neurons(self):
        # have to call Config here because transgenic lines doesn't exist
        self.config = Config(name=self.name,
                             imports=[f'NIFRAW:{self.branch}/ttl/generated/allen-transgenic-lines.ttl'],
                             prefixes=self.prefixes,
                             branch=self.branch,
                             sources=tuple(),  # TODO insert the link to the query...
                             source_file=relative_path(__file__))

        for cell_line in self.neuron_data:
            NeuronACT(*self.build_phenotypes(cell_line))

        print(sorted(self.tag_names))
        NeuronACT.write()
        NeuronACT.write_python()

    def build_transgenic_lines(self):
        """
        init class     |  "transgenic_line_source_name":"stock_number" a Class
        add superClass |  rdfs:subClassOf ilxtr:transgenicLine
        add *order*    |  ilxtr:useObjectProperty ilxtr:<order>
        add name       |  rdfs:label "name"
        add def        |  definition: "description"
        add transtype  |  rdfs:hasTransgenicType "transgenic_line_type_name"
        """

        triples = []
        for cell_line in self.neuron_data:
            for tl in cell_line['donor']['transgenic_lines']:
                _id = tl['stock_number'] if tl['stock_number'] else tl['id']
                prefix = tl['transgenic_line_source_name']
                line_type = tl['transgenic_line_type_name']
                if prefix not in ['JAX', 'MMRRC', 'AIBS']:
                    print(tc.red('WARNING:'), 'unknown prefix', prefix, json.dumps(tl, indent=4))
                    continue
                elif prefix == 'AIBS':
                    prefix = 'AllenTL'

                _class = self.ns[prefix][str(_id)]
                triples.append((_class, rdf.type, owl.Class))
                triples.append((_class, rdfs.label, rdflib.Literal(tl['name'])))
                triples.append((_class, definition, rdflib.Literal(tl['description'])))
                triples.append((_class, rdfs.subClassOf, ilxtr.transgenicLine))
                triples.append((_class, ilxtr.hasTransgenicType, ilxtr[line_type + 'Line']))

        # TODO aspects.ttl?
        transgenic_lines = simpleOnt(filename='allen-transgenic-lines',
                                     path='ttl/generated/',
                                     prefixes=self.prefixes,
                                     triples=triples,
                                     comment='Allen transgenic lines for cell types',
                                     branch=self.branch)

        transgenic_lines._graph.write()


def main(args={o.name:o.value for o in parse_defaults(__doc__)}):
    #print(args)
    if not args['--refresh'] and args['--input'] and Path(args['--input']).exists():
        with open(args['--input'], 'rt') as f:
            input = json.load(f)
    else:
        response = requests.get('http://api.brain-map.org/api/v2/data/query.json?criteria='
                                'model::Specimen,rma::criteria,[is_cell_specimen$eq%27true%27],'
                                'products[name$eq%27Mouse%20Cell%20Types%27],'
                                'rma::include,structure,donor(transgenic_lines),'
                                'specimen_tags,cell_soma_locations,rma::options[num_rows$eqall]')
        input = response.json()['msg']
        with open(args['--input'], 'wt') as f:
            json.dump(input, f, indent=4)

    act = AllenCellTypes(input, args['--output'])
    act.build_transgenic_lines()
    act.build_neurons()
    return act.config


if __name__ == '__main__':
    args = docopt(__doc__, version='0.0.4')
    main(args)
else:
    config = main()



#!/usr/bin/env python3.6

from pathlib import Path
from pyontutils.utils import rowParse, relative_path
from pyontutils.namespaces import ilxtr
from neurondm.lang import *
from neurondm.phenotype_namespaces import BBP
from IPython import embed

class NeuronMarkram2015(NeuronEBM):
    owlClass = ilxtr.NeuronMarkram2015
    shortname = 'Markram2015'

config = Config('markram-2015', source_file=relative_path(__file__))

with BBP:
    context = Neuron(Rat, S1, INT, GABA)

class table1(rowParse):
    citation = 'Markram et al Cell 2015'
    pmid = 'PMID:26451489'
    _sep = '|'

    def Morphological_type(self, value):
        syn, abrv = value.split(' (')
        syn = syn.strip()
        abrv = abrv.rstrip(')').strip()
        # print((syn, abrv))
        self._mtype = BBP[abrv]

        return self._mtype

    def Other_morphological_classifications(self, value):
        return  #  skipping all of this for now due to bitufted nonsense etc
        values = value.split(self._sep)
        output = []
        callbacks = []

        for v in values:
            if '/' in v:
                prefix, a_b = v.split(' ')
                a, b = a_b.split('/')
                #output.append(prefix + ' ' + a)
                #output.append(prefix + ' ' + b)
            else:
                prefix = v.rstrip('cell').strip()
                #output.append(v)

            label = prefix + ' phenotype'
            output.append(label)

        for v in output:
            self.graph.add_trip(self._morpho_parent_id, 'NIFRID:synonym', v)

    def Predominantly_expressed_Ca2_binding_proteins_and_peptides(self, value):
        p_edge = ilxtr.hasExpressionPhenotype
        with BBP:
            p_map = {
                'CB':CB,
                'PV':PV,
                'CR':CR,
                'NPY':NPY,
                'VIP':VIP,
                'SOM':SOM,
            }

        NEGATIVE = False
        POSITIVE = True  # FIXME this requires more processing prior to dispatch...
        e_edge = ''
        e_map = {
            '-':NEGATIVE,
            '+':POSITIVE,
            '++':POSITIVE,
            '+++':POSITIVE,
        }
        NONE = 0
        LOW = 1
        MED = 2
        HIGH = 3
        s_map = {
            '-':NONE,
            '+':LOW,
            '++':MED,
            '+++':HIGH,
        }

        values = value.split(self._sep)
        self._moltypes = []
        for v in values:
            abrv, score_paren = v.split(' (')
            score = score_paren.rstrip(')')
            molecule = BBP[abrv] #p_map[abrv]
            exists = e_map[score]
            score = s_map[score]
            if exists:
                self._moltypes.append(molecule)
            else:
                self._moltypes.append(NegPhenotype(molecule))

        return self._moltypes

    def Electrical_types(self, value):  # FIXME these are mutually exclusive types, so they force the creation of subClasses so we can't apply?
        with BBP:
            e_map = {
                'b':b,
                'c':c,
                'd':d,
            }
            l_map = {
                'AC':AC,
                'NAC':NAC,
                'STUT':STUT,
                'IR':IR,
            }

        values = value.split(self._sep)
        self._etypes = []
        for v in values:
            early_late, score_pct_paren = v.split(' (')
            score = int(score_pct_paren.rstrip('%)'))
            e_name, l_name = early_late[0], early_late[1:]
            #early, late = e_map[e_name], l_map[l_name]
            early, late = BBP[e_name], BBP[l_name]
            lpe = LogicalPhenotype(AND, early, late)
            self._etypes.append(lpe)

        return self._etypes

    def Other_electrical_classifications(self, value):
        with BBP:
            valid_mappings = {'Fast spiking':FS,
                              'Non-fast spiking':NegPhenotype(FS),  # only in this very limited context
                              'Regular spiking non-pyramidal':RSNP}

        values = value.split(self._sep)
        self._other_etypes = []
        for v in values:
            if v in valid_mappings:
                self._other_etypes.append(valid_mappings[v])

        return self._other_etypes

    def _row_post(self):
        with context:
            for etype in self._etypes:
                NeuronMarkram2015(etype, self._mtype, *self._other_etypes, *self._moltypes)

    def _end(self):
        graphBase.write()
        graphBase.write_python()

def main():
    import csv
    from pyontutils.config import devconfig
    with open(Path(devconfig.resources, '26451489 table 1.csv').as_posix(), 'rt') as f:
        rows = [list(r) for r in zip(*csv.reader(f))]
    table1(rows)

main()



import rdflib  # FIXME decouple
import ontquery as oq
from hyputils.hypothesis import idFromShareLink, shareLinkFromId
from pyontutils.sheets import update_sheet_values, get_note, Sheet
from pyontutils.scigraph import Vocabulary
from pyontutils.namespaces import ilxtr, TEMP, definition
from pyontutils.closed_namespaces import rdfs, rdf
from neurondm import NeuronCUT, Config, Phenotype, LogicalPhenotype
from neurondm.models.cuts import make_cut_id, fixname
from neurondm.core import log, OntId, OntTerm


def normalizeDoi(doi):
    if 'http' in doi:
        doi = '10.' + doi.split('.org/10.', 1)[-1]
    elif doi.startswith('doi:'):
        doi = doi.strip('doi:')
    elif doi.startswith('DOI:'):
        doi = doi.strip('DOI:')
    return doi


def select_by_curie_rank(results):
    ranking = 'CHEBI', 'UBERON', 'PR', 'NCBIGene', 'NCBITaxon', 'GO', 'SAO', 'NLXMOL'
    def key(result):
        if 'curie' in result:
            curie = result['curie']
        else:
            return len(results) * 3

        prefix, _ = curie.split(':')
        if prefix in ranking:
            try:
                return ranking.index(result['curie'])
            except ValueError:
                return len(results) + 1
        else:
            return len(results) * 2

    return sorted(results, key=key)[0]


def process_note(raw_note):
    if raw_note is None:
        return None
    p = ilxtr.literatureCitation
    for bit in  (b.strip() for b in raw_note.split('\n') if b.strip()):
        maybe_hypothesis = idFromShareLink(bit)
        if maybe_hypothesis:
            # TODO getDocInfoFromHypothesisId(maybe_hypothesis)
            yield p, rdflib.URIRef(shareLinkFromId(maybe_hypothesis))
        elif 'doi:' in bit or 'DOI:' in bit or 'doi.org' in bit:
            yield p, rdflib.URIRef('https://doi.org/' + normalizeDoi(bit))
        elif bit.startswith('http'):  # TODO parse the other things
            yield p, rdflib.URIRef(bit)
        else:
            yield p, rdflib.Literal(bit)  # FIXME cull editorial notes


def sheet_to_neurons(values, notes_index, expect_pes):
    # TODO import existing ids to register by label
    sgv = Vocabulary()
    e_config = Config('common-usage-types')
    e_config.load_existing()
    query = oq.OntQuery(oq.plugin.get('rdflib')(e_config.core_graph), instrumented=OntTerm)
    # FIXME clear use case for the remaining bound to whatever query produced it rather
    # than the other way around ... how to support this use case ...
    existing = {str(n.origLabel):n for n in e_config.neurons()}
    def convert_header(header):
        if header.startswith('has'):  # FIXME use a closed namespace
            return ilxtr[header]
        else:
            return None

    def convert_other(header):
        if header == 'label':
            return rdfs.label
        elif header == 'curie':
            return rdf.type
        elif header == 'definition':
            return definition
        else:
            header = header.replace(' ', '_')
            return TEMP[header]  # FIXME

    def mapCell(cell, syns=False):
        search_prefixes = ('UBERON', 'CHEBI', 'PR', 'NCBITaxon', 'NCBIGene', 'ilxtr', 'NIFEXT', 'SAO', 'NLXMOL',
                           'BIRNLEX',)

        if ':' in cell and ' ' not in cell:
            log.debug(cell)
            if 'http' in cell:
                if cell.startswith('http'):
                    t = OntTerm(iri=cell)
                else:
                    return None, None  # garbage with http inline
            else:
                t = OntTerm(cell, exclude_prefix=('FMA',))  # FIXME need better error message in ontquery

            return t.u, t.label

        result = [r for r in sgv.findByTerm(cell, searchSynonyms=syns, prefix=search_prefixes)
                  if not r['deprecated']]
        #printD(cell, result)
        if not result:
            log.debug(f'{cell}')
            maybe = list(query(label=cell, exclude_prefix=('FMA',)))
            if maybe:
                qr = maybe[0]
                return qr.OntTerm.u, qr.label
            elif not syns:
                return mapCell(cell, syns=True)
            else:
                return None, None
        elif len(result) > 1:
            #printD('WARNING', result)
            result = select_by_curie_rank(result)
        else:
            result = result[0]

        return rdflib.URIRef(result['iri']), result['labels'][0]

    def lower_check(label, cell):
        return label not in cell and label.lower() not in cell.lower()  # have to handle comma sep case

    lnlu = {v:k for k, v in LogicalPhenotype.local_names.items()}
    def convert_cell(cell_or_comma_sep):
        #printD('CONVERTING', cell_or_comma_sep)
        for cell_w_junk in cell_or_comma_sep.split(','):  # XXX WARNING need a way to alter people to this
            cell = cell_w_junk.strip()
            if cell.startswith('(OR') or cell.startswith('(AND'):
                start, *middle, end = cell.split('" "')
                OPoperator, first = start.split(' "')
                operator = OPoperator[1:]
                operator = lnlu[operator]
                last, CP = end.rsplit('"')
                iris, labels = [], []
                for term in (first, *middle, last):
                    iri, label = mapCell(term)
                    if label is None:
                        label = cell_or_comma_sep
                    iris.append(iri)
                    labels.append(label)

                yield (operator, *iris), tuple(labels)

            else:
                iri, label = mapCell(cell)
                if label is None:
                    yield iri, cell_or_comma_sep  # FIXME need a way to handle this that doesn't break things?
                else:
                    yield iri, label

    config = Config('cut-roundtrip')
    skip = 'alignment label',
    headers, *rows = values
    errors = []
    new = []
    release = []
    for i, neuron_row in enumerate(rows):
        id = None
        label_neuron  = None
        definition_neuron = None
        synonyms_neuron = None
        current_neuron = None
        phenotypes = []
        do_release = False
        predicate_notes = {}
        object_notes = {}
        other_notes = {}
        wat = {}
        for j, (header, cell) in enumerate(zip(headers, neuron_row)):
            notes = list(process_note(get_note(i + 1, j, notes_index)))  # + 1 since headers is removed
            if notes and not header.startswith('has'):
                _predicate = convert_other(header)
                if cell:
                    _object = rdflib.Literal(cell)  # FIXME curies etc.
                else:
                    _object = rdf.nil
                other_notes[_predicate, _object] = notes

            if header == 'curie':
                id = OntId(cell).u if cell else None
                continue
            elif header == 'label':
                label_neuron = cell
                if cell in existing:
                    current_neuron = existing[cell]
                elif cell:
                    # TODO
                    new.append(cell)
                else:
                    raise ValueError(cell)  # wat
                continue
            elif header == 'Status':
                # TODO
                if cell == 'Yes':
                    do_release = True
                elif cell == 'Maybe':
                    pass
                elif cell == 'Not yet':
                    pass
                elif cell == 'Delete':
                    pass
                else:
                    pass

                continue
            elif header == 'PMID':
                # TODO
                continue
            elif header == 'Other reference':
                # TODO
                continue
            elif header == 'Other label':
                # TODO
                continue
            elif header == 'definition':
                continue  # FIXME single space differences between the spreadsheet and the source

                if cell:
                    definition_neuron = rdflib.Literal(cell)

                continue

            elif header == 'synonyms':
                if cell:
                    synonyms_neuron = [rdflib.Literal(s.strip())
                                    # FIXME bare comma is extremely dangerous
                                    for s in cell.split(',')]

                continue
            elif header in skip:
                continue

            objects = []
            if cell:
                predicate = convert_header(header)
                if predicate is None:
                    log.debug(f'{(header, cell, notes)}')

                for object, label in convert_cell(cell):
                    if isinstance(label, tuple):  # LogicalPhenotype case
                        _err = []
                        for l in label:
                            if lower_check(l, cell):
                                _err.append((cell, label))
                        if _err:
                            errors.extend(_err)
                        else:
                            objects.append(object)
                    elif lower_check(label, cell):
                        errors.append((cell, label))
                    elif str(id) == object:
                        errors.append((header, cell, object, label))
                        object = None
                    else:
                        objects.append(object)

                if notes:
                    # FIXME this is a hack to only attach to the last value
                    # since we can't distinguish at the moment
                    wat[predicate, object] = notes
                    if object is not None:
                        # object aka iri can be none if we don't find anything
                        object_notes[object] = notes
                    else:
                        predicate_notes[predicate] = notes
                        # FIXME it might also be simpler in some cases
                        # to have this be object_notes[object] = notes
                        # because we are much less likely to have the same
                        # phenotype appear attached to the different dimensions

                        # FIXME comma sep is weak here because the
                        # reference is technically ambiguous
                        # might be an argument for the denormalized form ...
                        # or perhaps having another sheet for cases like that

            else:
                continue

            if predicate and objects:
                for object in objects:  # FIXME has layer location phenotype
                    if isinstance(object, tuple):
                        op, *rest = object
                        pes = (Phenotype(r, predicate) for r in rest)  # FIXME nonhomogenous phenotypes
                        phenotypes.append(LogicalPhenotype(op, *pes))
                    elif object:
                        phenotypes.append(Phenotype(object, predicate))
                    else:
                        errors.append((object, predicate, cell))
            elif objects:
                errors.append((header, objects))
            else:
                errors.append((header, cell))
            # translate header -> predicate
            # translate cell value to ontology id

        if current_neuron and phenotypes:
            # TODO merge current with changes
            # or maybe we just replace since all the phenotypes should be there?
            log.debug(phenotypes)
            if id is not None:
                log.debug(f'{(id, bool(id))}')

            elif label_neuron:
                id = make_cut_id(label_neuron)

            if id not in expect_pes:
                log.error(f'{id!r} not in cuts!?')
                continue

            if expect_pes[id] != len(phenotypes):
                log.error(f'{id!r} failed roundtrip {len(phenotypes)} != {expect_pes[id]}')
                continue

            neuron = NeuronCUT(*phenotypes, id_=id, label=label_neuron,
                               override=bool(id) or bool(label_neuron))
            neuron.adopt_meta(current_neuron)
            # FIXME occasionally this will error?!
        else:
            continue  # FIXME this polutes everything ???
            fn = fixname(label_neuron)
            if not phenotypes and i:  # i skips header
                errors.append((i, neuron_row))  # TODO special review for phenos but not current
                phenotypes = Phenotype('TEMP:phenotype/' + fn),

            neuron = NeuronCUT(*phenotypes,
                               id_=make_cut_id(label_neuron),
                               label=label_neuron, override=True)

        # update the meta if there were any changes
        if definition_neuron is not None:
            neuron.definition = definition_neuron

        if synonyms_neuron is not None:
            neuron.synonyms = synonyms_neuron

        try:
            neuron.batchAnnotateByObject(object_notes)
            neuron.batchAnnotate(other_notes)
        except AttributeError as e:
            #embed()
            log.exception(e) #'something very strage has happened\n', e)
            pass  # FIXME FIXME FIXME

        #neuron.batchAnnotateByPredicate(predicate_notes)  # TODO
        # FIXME doesn't quite work in this context, but there are other
        # cases where annotations to the general modality are still desireable
        # FIXME there may be no predicate? if the object fails to match?

        if do_release:
            release.append(neuron)

    return config, errors, new, release


class Cuts(Sheet):
    name = 'neurons-cut'


class CutsV1(Cuts):
    sheet_name = 'CUT V1.0'
    fetch_grid = True


def main():
    #from neurondm.models.cuts import main as cuts_main
    #cuts_config, *_ = cuts_main()
    from IPython import embed
    from neurondm.compiled.common_usage_types import config as cuts_config
    cuts_neurons = cuts_config.neurons()
    expect_pes = {n.id_:len(n.pes) for n in cuts_neurons}

    sheet = CutsV1()
    config, errors, new, release = sheet_to_neurons(sheet.values, sheet.notes_index, expect_pes)
    #sheet.show_notes()
    config.write_python()
    config.write()
    #config = Config(config.name)
    #config.load_existing()  # FIXME this is a hack to get get a load_graph
    from neurondm import Config, NeuronCUT
    release_config = Config('cut-release')
    [NeuronCUT(*n, id_=n.id_, label=n.origLabel, override=True).adopt_meta(n) for n in release]
    release_config.write_python()
    release_config.write()
    from neurondm.models.cuts import export_for_review
    review_rows = export_for_review(config, [], [], [], filename='cut-rt-test.csv', with_curies=True)
    from pyontutils.utils import byCol
    valuesC = byCol(sheet.values[1:],
                    header=[v.replace(' ', '_') for v in sheet.values[0]],
                    to_index=['label'])
    reviewC = byCol(review_rows[1:], header=[v.replace(' ', '_') for v in review_rows[0]], to_index=['label'])
    def grow(r):
        log.debug(r)
        # TODO implement on the object to allow joining on an index?
        # man this would be easier with sql >_< probably pandas too
        # but so many dependencies ... also diffing issues etc
        return valuesC.searchIndex('label', r.label)

    def key(field_value):
        field, value = field_value
        try:
            return valuesC.header._fields.index(field)  # TODO warn on field mismatch
        except ValueError as e:
            print('ERROR!!!!!!!!!!!', field, value)
            return None

    def replace(r, *cols):
        """ replace and reorder """
        # FIXME _super_ inefficient
        vrow = grow(r)
        for field, value in sorted(zip(r._fields, r), key=key):
            if field in cols:
                value = getattr(vrow, field)

            yield '' if value is None else value  # completely overwrite the sheet

    rows = [list(replace(r, 'Status', 'definition', 'synonyms', 'PMID')) for r in reviewC]
    #resp = update_sheet_values('neurons-cut', 'Roundtrip', rows)
    embed()

if __name__ == '__main__':
    main()



#!/usr/bin/env python3.6
from pathlib import Path
import rdflib
from pyontutils.core import makeGraph
from pyontutils.utils import relative_path
from pyontutils.config import devconfig
from pyontutils.namespaces import makePrefixes, TEMP
from neurondm import *
from neurondm.lang import *
from neurondm.core import MeasuredNeuron
from pyontutils.closed_namespaces import rdf, rdfs, owl
from IPython import embed


def main():
    # load in our existing graph
    # note: while it would be nice to allow specification of phenotypes to be decoupled
    # from insertion into the graph... maybe we could enable this, but it definitely seems
    # to break a number of nice features... and we would need the phenotype graph anyway
    Config('temporary-graph')
    EXISTING_GRAPH = graphBase.in_graph
    #EXISTING_GRAPH = rdflib.Graph()
    #graphBase.in_graph = EXISTING_GRAPH
    #graphBase.core_graph = EXISTING_GRAPH
    local_prefix = Path(devconfig.ontology_local_repo, 'ttl')
    sources = (f'{local_prefix}/NIF-Neuron-Defined.ttl',
               f'{local_prefix}/NIF-Neuron.ttl',
               f'{local_prefix}/NIF-Neuron-Phenotype.ttl',
               f'{local_prefix}/phenotype-core.ttl',
               f'{local_prefix}/phenotypes.ttl',
               f'{local_prefix}/hbp-special.ttl')
    for file in sources:
            EXISTING_GRAPH.parse(file, format='turtle')
    #EXISTING_GRAPH.namespace_manager.bind('PR', makePrefixes('PR')['PR'])

    #graphBase.core_graph = EXISTING_GRAPH
    #graphBase.out_graph = rdflib.Graph()
    graphBase.__import_name__ = 'neurondm.lang'
    graphBase._predicates = getPhenotypePredicates(EXISTING_GRAPH)

    g = makeGraph('merged', prefixes={k:str(v) for k, v in EXISTING_GRAPH.namespaces()}, graph=EXISTING_GRAPH)
    reg_neurons = list(g.g.subjects(rdfs.subClassOf, _NEURON_CLASS))
    tc_neurons = [_ for (_,) in g.g.query('SELECT DISTINCT ?match WHERE {?match rdfs:subClassOf+ %s}' % g.g.qname(_NEURON_CLASS))]
    def_neurons = g.get_equiv_inter(_NEURON_CLASS)

    nodef = sorted(set(tc_neurons) - set(def_neurons))
    og1 = MeasuredNeuron.out_graph = rdflib.Graph()  # there is only 1 out_graph at a time, load and switch

    mns = [MeasuredNeuron(id_=n) for n in nodef]
    mnsp = [n for n in mns if n.pes]
    graphBase.out_graph = rdflib.Graph()  # XXX NEVER DO THIS IT IS EVIL ZALGO WILL EAT YOUR FACE
    graphBase.ng.g = graphBase.out_graph
    # and he did, had to swtich to graphBase for exactly this reason >_<
    dns = [Neuron(id_=n) for n in sorted(def_neurons)]
    #dns += [Neuron(*m.pes) if m.pes else m.id_ for m in mns]
    dns += [Neuron(*m.pes) for m in mns if m.pes]

    # reset everything for export
    config = Config('phenotype-direct', source_file=relative_path(__file__))
    #Neuron.out_graph = graphBase.out_graph  # each subclass of graphBase has a distinct out graph IF it was set manually
    #Neuron.out_graph = rdflib.Graph()
    #ng = makeGraph('', prefixes={}, graph=Neuron.out_graph)
    #ng.filename = Neuron.ng.filename
    Neuron.mro()[1].existing_pes = {}  # wow, new adventures in evil python patterns mro()[1]
    dns = [Neuron(*d.pes) for d in set(dns)]  # TODO remove the set and use this to test existing bags?
    #from neurons.lang import WRITEPYTHON
    #WRITEPYTHON(sorted(dns))
    #ng.add_ont(TEMP['defined-neurons'], 'Defined Neurons', 'NIFDEFNEU',
               #'VERY EXPERIMENTAL', '0.0.0.1a')
    #ng.add_trip(TEMP['defined-neurons'], owl.imports, rdflib.URIRef('file:///home/tom/git/NIF-Ontology/ttl/phenotype-core.ttl'))
    #ng.add_trip(TEMP['defined-neurons'], owl.imports, rdflib.URIRef('file:///home/tom/git/NIF-Ontology/ttl/phenotypes.ttl'))
    #ng.write()
    ontinfo = (
        (Neuron.ng.ontid, rdf.type, owl.Ontology),
        (Neuron.ng.ontid, rdfs.label, rdflib.Literal('phenotype direct neurons')),
        (Neuron.ng.ontid, rdfs.comment, rdflib.Literal('Neurons derived directly from phenotype definitions')),
    )
    [Neuron.out_graph.add(t) for t in ontinfo]
    Neuron.write()
    Neuron.write_python()
    bads = [n for n in Neuron.ng.g.subjects(rdf.type, owl.Class)
            if len(list(Neuron.ng.g.predicate_objects(n))) == 1]
    if __name__ == '__main__':
        embed()

    return config

config = main()



import pkgutil
from pathlib import Path

# __path__ fails in nosetests for reasons I do not understand
thispath = [Path(__file__).parent.as_posix()]
__all__ = [mod for _, mod, is_pkg in pkgutil.walk_packages(thispath)]



#!/usr/bin/env python3
import rdflib
import ontquery
#from pyontutils.core import OntId, OntTerm
from pyontutils.utils import relative_path
from pyontutils.namespaces import makePrefixes, makeNamespaces
from pyontutils.namespaces import NIFRID, ilxtr, hasRole, definition
from pyontutils.closed_namespaces import rdf, rdfs, owl
from neurondm.lang import *
from neurondm import *
from neurondm.phenotype_namespaces import *
from IPython import embed

config = Config('huang-2017',
                imports=['NIFRAW:neurons/ttl/generated/neurons/phenotype-direct.ttl'],
                source_file=relative_path(__file__))
OntTerm.query.add(ontquery.plugin.get('rdflib')(Neuron.core_graph, OntId))


class NeuronHuang2017(NeuronEBM):
    owlClass = ilxtr.NeuronHuang2017
    shortname = 'Huang2017'


Neuron, Neuron_ = NeuronHuang2017, Neuron
dex = 'ilxtr:hasDriverExpressionPhenotype'


class Genes(LocalNameManager):

    # FIXME ilxtr.hasRNAExpressionPhenotype
    #VIP = Phenotype('PR:000017299', 'ilxtr:hasExpressionPhenotype')
    #CCK = Phenotype('PR:000005110', 'ilxtr:hasExpressionPhenotype')
    #SST = Phenotype('PR:000015665', 'ilxtr:hasExpressionPhenotype')
    #CR = Phenotype('PR:000004968', 'ilxtr:hasExpressionPhenotype')
    #PV = Phenotype('PR:000013502', 'ilxtr:hasExpressionPhenotype')

    # cre lines
    VIP = Phenotype('ilxtr:VIP-flp', dex, label='VIP-flp', override=True)
    CCK = Phenotype('ilxtr:CCK-cre', dex, label='CCK-cre', override=True)
    SST = Phenotype('ilxtr:SST-flp', dex, label='SST-flp', override=True)
    CR = Phenotype('ilxtr:CR-cre', dex, label='CR-cre', override=True)
    PV = Phenotype('ilxtr:PV-cre', dex, label='PV-cre', override=True)
    NOS1 = Phenotype('ilxtr:NOS1-creER', dex, label='NOS1-creER', override=True)
    Nkx2_1 = Phenotype('ilxtr:Nkx2.1-creER', dex, label='Nkx2.1-creER', override=True)
    Nkx2_1flp = Phenotype('ilxtr:Nkx2.1-flp', dex, label='Nxk2.1-flp', override=True)

    # Actual genes

    Vip = Phenotype('NCBIGene:22353', ilxtr.hasExpressionPhenotype)
    Cck = Phenotype('NCBIGene:12424', ilxtr.hasExpressionPhenotype)
    Sst = Phenotype('NCBIGene:20604', ilxtr.hasExpressionPhenotype)
    Calb2 = Phenotype('NCBIGene:12308', ilxtr.hasExpressionPhenotype)
    Pvalb = Phenotype('NCBIGene:19293', ilxtr.hasExpressionPhenotype)
    Nos1 = Phenotype('NCBIGene:18125', 'ilxtr:hasExpressionPhenotype')

    # cre equivalents where appropriate
    PVBCEq = LogicalPhenotype(AND, Pvalb)
    CHCEq = LogicalPhenotype(AND)  # pretty sure this doesn't have any
    CCKCEq = LogicalPhenotype(AND, Vip, Cck)
    MNCEq = LogicalPhenotype(AND, Sst, Calb2)
    ISCEq = LogicalPhenotype(AND, Vip, Calb2)
    LPCEq = LogicalPhenotype(AND, Sst, Nos1)

    # batch replace with ncbigene
    Adm = Phenotype('NCBIGene:11535', 'ilxtr:hasExpressionPhenotype', label='Adm')
    Calca = Phenotype('NCBIGene:12310', 'ilxtr:hasExpressionPhenotype', label='Calca')
    Chrm2 = Phenotype('NCBIGene:243764', 'ilxtr:hasExpressionPhenotype', label='Chrm2')
    Cnr1 = Phenotype('NCBIGene:12801', 'ilxtr:hasExpressionPhenotype', label='Cnr1')
    Cort = Phenotype('NCBIGene:12854', 'ilxtr:hasExpressionPhenotype', label='Cort')
    Cplx3 = Phenotype('NCBIGene:235415', 'ilxtr:hasExpressionPhenotype', label='Cplx3')
    Crh = Phenotype('NCBIGene:12918', 'ilxtr:hasExpressionPhenotype', label='Crh')
    Edn3 = Phenotype('NCBIGene:13616', 'ilxtr:hasExpressionPhenotype', label='Edn3')
    Htr3a = Phenotype('NCBIGene:15561', 'ilxtr:hasExpressionPhenotype', label='Htr3a')
    Igf1 = Phenotype('NCBIGene:16000', 'ilxtr:hasExpressionPhenotype', label='Igf1')
    Kcnmb2 = Phenotype('NCBIGene:72413', 'ilxtr:hasExpressionPhenotype', label='Kcnmb2')
    Nefh = Phenotype('NCBIGene:380684', 'ilxtr:hasExpressionPhenotype', label='Nefh')
    Nppc = Phenotype('NCBIGene:18159', 'ilxtr:hasExpressionPhenotype', label='Nppc')
    Oxtr = Phenotype('NCBIGene:18430', 'ilxtr:hasExpressionPhenotype', label='Oxtr')
    Penk = Phenotype('NCBIGene:18619', 'ilxtr:hasExpressionPhenotype', label='Penk')
    Pnoc = Phenotype('NCBIGene:18155', 'ilxtr:hasExpressionPhenotype', label='Pnoc')
    Syt6 = Phenotype('NCBIGene:54524', 'ilxtr:hasExpressionPhenotype', label='Syt6')
    Tac1 = Phenotype('NCBIGene:21333', 'ilxtr:hasExpressionPhenotype', label='Tac1')
    Tac2 = Phenotype('NCBIGene:21334', 'ilxtr:hasExpressionPhenotype', label='Tac2')
    Tacr1 = Phenotype('NCBIGene:21336', 'ilxtr:hasExpressionPhenotype', label='Tacr1')
    Trpc6 = Phenotype('NCBIGene:22068', 'ilxtr:hasExpressionPhenotype', label='Trpc6')
    Wnt5a = Phenotype('NCBIGene:22418', 'ilxtr:hasExpressionPhenotype', label='Wnt5a')

    # batch 2
    Adcy1 = Phenotype('NCBIGene:432530', 'ilxtr:hasExpressionPhenotype', label='Adcy1', override=True)
    Adcy2 = Phenotype('NCBIGene:210044', 'ilxtr:hasExpressionPhenotype', label='Adcy2', override=True)
    Adcy8 = Phenotype('NCBIGene:11514', 'ilxtr:hasExpressionPhenotype', label='Adcy8', override=True)
    Adcy9 = Phenotype('NCBIGene:11515', 'ilxtr:hasExpressionPhenotype', label='Adcy9', override=True)
    Adra1b = Phenotype('NCBIGene:11548', 'ilxtr:hasExpressionPhenotype', label='Adra1b', override=True)
    Arhgef10 = Phenotype('NCBIGene:234094', 'ilxtr:hasExpressionPhenotype', label='Arhgef10', override=True)
    Cckbr = Phenotype('NCBIGene:12426', 'ilxtr:hasExpressionPhenotype', label='Cckbr', override=True)
    Chrm3 = Phenotype('NCBIGene:12671', 'ilxtr:hasExpressionPhenotype', label='Chrm3', override=True)
    Chrna4 = Phenotype('NCBIGene:11438', 'ilxtr:hasExpressionPhenotype', label='Chrna4', override=True)
    Chst1 = Phenotype('NCBIGene:76969', 'ilxtr:hasExpressionPhenotype', label='Chst1', override=True)
    Cox6c = Phenotype('NCBIGene:12864', 'ilxtr:hasExpressionPhenotype', label='Cox6c', override=True)
    Cplx1 = Phenotype('NCBIGene:12889', 'ilxtr:hasExpressionPhenotype', label='Cplx1', override=True)
    Cplx2 = Phenotype('NCBIGene:12890', 'ilxtr:hasExpressionPhenotype', label='Cplx2', override=True)
    Esrrg = Phenotype('NCBIGene:26381', 'ilxtr:hasExpressionPhenotype', label='Esrrg', override=True)
    Fgf9 = Phenotype('NCBIGene:14180', 'ilxtr:hasExpressionPhenotype', label='Fgf9', override=True)
    Gpr88 = Phenotype('NCBIGene:64378', 'ilxtr:hasExpressionPhenotype', label='Gpr88', override=True)
    Grin3a = Phenotype('NCBIGene:242443', 'ilxtr:hasExpressionPhenotype', label='Grin3a', override=True)
    Hcrtr1 = Phenotype('NCBIGene:230777', 'ilxtr:hasExpressionPhenotype', label='Hcrtr1', override=True)
    Hs3st4 = Phenotype('NCBIGene:628779', 'ilxtr:hasExpressionPhenotype', label='Hs3st4', override=True)
    Hs6st3 = Phenotype('NCBIGene:50787', 'ilxtr:hasExpressionPhenotype', label='Hs6st3', override=True)
    Htr2c = Phenotype('NCBIGene:15560', 'ilxtr:hasExpressionPhenotype', label='Htr2c', override=True)
    Inhbb = Phenotype('NCBIGene:16324', 'ilxtr:hasExpressionPhenotype', label='Inhbb', override=True)
    Kcnmb4 = Phenotype('NCBIGene:58802', 'ilxtr:hasExpressionPhenotype', label='Kcnmb4', override=True)
    Mef2c = Phenotype('NCBIGene:17260', 'ilxtr:hasExpressionPhenotype', label='Mef2c', override=True)
    Npy1r = Phenotype('NCBIGene:18166', 'ilxtr:hasExpressionPhenotype', label='Npy1r', override=True)
    Npy2r = Phenotype('NCBIGene:18167', 'ilxtr:hasExpressionPhenotype', label='Npy2r', override=True)
    Nrtn = Phenotype('NCBIGene:18188', 'ilxtr:hasExpressionPhenotype', label='Nrtn', override=True)
    Opn3 = Phenotype('NCBIGene:13603', 'ilxtr:hasExpressionPhenotype', label='Opn3', override=True)
    Pde1a = Phenotype('NCBIGene:18573', 'ilxtr:hasExpressionPhenotype', label='Pde1a', override=True)
    Pde2a = Phenotype('NCBIGene:207728', 'ilxtr:hasExpressionPhenotype', label='Pde2a', override=True)
    Pde4b = Phenotype('NCBIGene:18578', 'ilxtr:hasExpressionPhenotype', label='Pde4b', override=True)
    Pde5a = Phenotype('NCBIGene:242202', 'ilxtr:hasExpressionPhenotype', label='Pde5a', override=True)
    Pde7b = Phenotype('NCBIGene:29863', 'ilxtr:hasExpressionPhenotype', label='Pde7b', override=True)
    Pde11a = Phenotype('NCBIGene:241489', 'ilxtr:hasExpressionPhenotype', label='Pde11a', override=True)
    Pparg = Phenotype('NCBIGene:19016', 'ilxtr:hasExpressionPhenotype', label='Pparg', override=True)
    Prkg1 = Phenotype('NCBIGene:19091', 'ilxtr:hasExpressionPhenotype', label='Prkg1', override=True)
    Prkg2 = Phenotype('NCBIGene:19092', 'ilxtr:hasExpressionPhenotype', label='Prkg2', override=True)
    Prok2 = Phenotype('NCBIGene:50501', 'ilxtr:hasExpressionPhenotype', label='Prok2', override=True)
    Pthlh = Phenotype('NCBIGene:19227', 'ilxtr:hasExpressionPhenotype', label='Pthlh', override=True)
    Ptn = Phenotype('NCBIGene:19242', 'ilxtr:hasExpressionPhenotype', label='Ptn', override=True)
    Rgs10 = Phenotype('NCBIGene:67865', 'ilxtr:hasExpressionPhenotype', label='Rgs10', override=True)
    Rgs12 = Phenotype('NCBIGene:71729', 'ilxtr:hasExpressionPhenotype', label='Rgs12', override=True)
    Rgs16 = Phenotype('NCBIGene:19734', 'ilxtr:hasExpressionPhenotype', label='Rgs16', override=True)
    Rgs8 = Phenotype('NCBIGene:67792', 'ilxtr:hasExpressionPhenotype', label='Rgs8', override=True)
    Rln1 = Phenotype('NCBIGene:19773', 'ilxtr:hasExpressionPhenotype', label='Rln1', override=True)
    Rspo1 = Phenotype('NCBIGene:192199', 'ilxtr:hasExpressionPhenotype', label='Rspo1', override=True)
    Slc7a3 = Phenotype('NCBIGene:11989', 'ilxtr:hasExpressionPhenotype', label='Slc7a3', override=True)
    Slit2 = Phenotype('NCBIGene:20563', 'ilxtr:hasExpressionPhenotype', label='Slit2', override=True)
    Slit3 = Phenotype('NCBIGene:20564', 'ilxtr:hasExpressionPhenotype', label='Slit3', override=True)
    Syt10 = Phenotype('NCBIGene:54526', 'ilxtr:hasExpressionPhenotype', label='Syt10', override=True)
    Syt2 = Phenotype('NCBIGene:20980', 'ilxtr:hasExpressionPhenotype', label='Syt2', override=True)
    Syt4 = Phenotype('NCBIGene:20983', 'ilxtr:hasExpressionPhenotype', label='Syt4', override=True)
    Syt5 = Phenotype('NCBIGene:53420', 'ilxtr:hasExpressionPhenotype', label='Syt5', override=True)
    Syt7 = Phenotype('NCBIGene:54525', 'ilxtr:hasExpressionPhenotype', label='Syt7', override=True)
    Tgfb3 = Phenotype('NCBIGene:21809', 'ilxtr:hasExpressionPhenotype', label='Tgfb3', override=True)
    Trpc5 = Phenotype('NCBIGene:22067', 'ilxtr:hasExpressionPhenotype', label='Trpc5', override=True)
    Unc5a = Phenotype('NCBIGene:107448', 'ilxtr:hasExpressionPhenotype', label='Unc5a', override=True)
    Unc5b = Phenotype('NCBIGene:107449', 'ilxtr:hasExpressionPhenotype', label='Unc5b', override=True)
    Unc5d = Phenotype('NCBIGene:210801', 'ilxtr:hasExpressionPhenotype', label='Unc5d', override=True)
    Vamp1 = Phenotype('NCBIGene:22317', 'ilxtr:hasExpressionPhenotype', label='Vamp1', override=True)
    Vipr1 = Phenotype('NCBIGene:22354', 'ilxtr:hasExpressionPhenotype', label='Vipr1', override=True)
    Wnt2 = Phenotype('NCBIGene:22413', 'ilxtr:hasExpressionPhenotype', label='Wnt2', override=True)

    # peptides
    #Tac1 = Phenotype('ilxtr:Tac1', 'ilxtr:hasExpressionPhenotype')
    #Adm = Phenotype('ilxtr:Adm', 'ilxtr:hasExpressionPhenotype')
    Rspn = Phenotype('ilxtr:Rspn', 'ilxtr:hasExpressionPhenotype')
    PVBCPep = LogicalPhenotype(AND, Tac1, Adm, Rspn)

    #Pthlh = Phenotype('ilxtr:Pthlh', 'ilxtr:hasExpressionPhenotype')
    #Tgfb3 = Phenotype('ilxtr:Tgfb3', 'ilxtr:hasExpressionPhenotype')
    #Fgf9 = Phenotype('ilxtr:Fgf91', 'ilxtr:hasExpressionPhenotype')
    CHCPep = LogicalPhenotype(AND, Pthlh, Tgfb3, Fgf9) 

    #Igf1 = Phenotype('ilxtr:Igf1', 'ilxtr:hasExpressionPhenotype')
    #Edn3 = Phenotype('ilxtr:Edn3', 'ilxtr:hasExpressionPhenotype')
    #Prok2 = Phenotype('ilxtr:Prok2', 'ilxtr:hasExpressionPhenotype')
    #Pnoc = Phenotype('ilxtr:Pnoc', 'ilxtr:hasExpressionPhenotype')
    #Crh = Phenotype('ilxtr:Crh', 'ilxtr:hasExpressionPhenotype')
    #Tac2 = Phenotype('ilxtr:Tac2', 'ilxtr:hasExpressionPhenotype')
    CCKCPep = LogicalPhenotype(AND, Vip, Cck, Igf1, Edn3, Prok2, Pnoc, Crh, Tac2)

    #Inhbb = Phenotype('ilxtr:Inhbb', 'ilxtr:hasExpressionPhenotype')
    #Nppc = Phenotype('ilxtr:Nppc', 'ilxtr:hasExpressionPhenotype')
    MNCPep = LogicalPhenotype(AND, Sst, Inhbb, Nppc)

    #Rspo1 = Phenotype('ilxtr:Rspo1', 'ilxtr:hasExpressionPhenotype')
    #Wnt5a = Phenotype('ilxtr:Wnt5aae1', 'ilxtr:hasExpressionPhenotype')
    #Nrtn = Phenotype('ilxtr:Nrtn', 'ilxtr:hasExpressionPhenotype')
    ISCPep = LogicalPhenotype(AND, Vip, Cck, Rspo1, Wnt5a, Nrtn)

    #Ptn = Phenotype('ilxtr:Ptn', 'ilxtr:hasExpressionPhenotype')
    #Wnt2 = Phenotype('ilxtr:Wnt2', 'ilxtr:hasExpressionPhenotype')
    #Rln1 = Phenotype('ilxtr:Rln1', 'ilxtr:hasExpressionPhenotype')
    #Penk = Phenotype('ilxtr:Penk', 'ilxtr:hasExpressionPhenotype')
    #Calca = Phenotype('ilxtr:Calca', 'ilxtr:hasExpressionPhenotype')
    #Cort = Phenotype('ilxtr:Cort', 'ilxtr:hasExpressionPhenotype')
    LPCPep = LogicalPhenotype(AND, Ptn, Wnt2, Rln1, Penk, Calca, Cort)

    # dendrites
    GluA1 = Phenotype('ilxtr:GluA1', 'ilxtr:hasExpressionPhenotype')
    GluA4 = Phenotype('ilxtr:GluA4', 'ilxtr:hasExpressionPhenotype')
    α1GABAaR = Phenotype('ilxtr:α1GABAaR', 'ilxtr:hasExpressionPhenotype')
    Kv3 = Phenotype('ilxtr:Kv3', 'ilxtr:hasExpressionPhenotype')
    α4GABAaR = Phenotype('ilxtr:α4GABAaR', 'ilxtr:hasExpressionPhenotype')
    δGABAaR = Phenotype('ilxtr:δGABAaR', 'ilxtr:hasExpressionPhenotype')
    #Cckbr = Phenotype('ilxtr:Cckbr', 'ilxtr:hasExpressionPhenotype')
    PVBCDend = LogicalPhenotype(AND, GluA1, GluA4, α1GABAaR, Kv3, α4GABAaR, δGABAaR, Cckbr)

    # TODO LOWER
    Lower = Phenotype('ilxtr:LowerExpression', 'ilxtr:hasPhenotypeModifier')
    Higher = Phenotype('ilxtr:HigherExpression', 'ilxtr:hasPhenotypeModifier')
    CHCDend = LogicalPhenotype(AND,
                               GluA1, GluA4, α1GABAaR, Kv3,
                               α4GABAaR, δGABAaR, Cckbr,
                               Lower) # FIXME bad model

    #Cnr1 = Phenotype('ilxtr:Cnr1', 'ilxtr:hasExpressionPhenotype')
    #Htr2c = Phenotype('ilxtr:Htr2c', 'ilxtr:hasExpressionPhenotype')
    #Htr3a = Phenotype('ilxtr:Htr3a', 'ilxtr:hasExpressionPhenotype')
    #Chrm3 = Phenotype('ilxtr:Chrm3', 'ilxtr:hasExpressionPhenotype')
    #Npy1r = Phenotype('ilxtr:Npy1r', 'ilxtr:hasExpressionPhenotype')
    #Vipr1 = Phenotype('ilxtr:Vipr1', 'ilxtr:hasExpressionPhenotype')
    CCKCDend = LogicalPhenotype(AND, Cnr1, Htr2c, Htr3a, Chrm3, Npy1r, Vipr1)

    # high level diverse iGluRs
    # low level few types GABAaRs
    # FIXME the helper graph is not included for query at the moment so label gen has bad values
    _GLUR = Phenotype('ilxtr:glutamateReceptor', 'ilxtr:hasExpressionPhenotype', label='GluR', override=True)
    _GABAR = Phenotype('ilxtr:GABAReceptor', 'ilxtr:hasExpressionPhenotype', label='GABAR', override=True)
    # need a clear statement for how to handle cases like this
    # can the current python implementation handle this?
    MNCDend = LogicalPhenotype(AND,
                     LogicalPhenotype(AND, _GABAR, Lower),
                     # intersectionOf(ilxtr.GABAReceptor, ilxtr.LowerExpression)
                     LogicalPhenotype(AND, _GLUR, Higher))
    #Chrna4 = Phenotype('ilxtr:Chrna4', 'ilxtr:hasExpressionPhenotype')
    #Adra1b = Phenotype('ilxtr:Adra1b', 'ilxtr:hasExpressionPhenotype')
    #Npy2r = Phenotype('ilxtr:Npy2r', 'ilxtr:hasExpressionPhenotype')
    ISCDend = LogicalPhenotype(AND, Htr2c, Htr3a, Chrm3, Chrna4, Adra1b, Npy2r)

    #Chrm2 = Phenotype('ilxtr:Chrm2', 'ilxtr:hasExpressionPhenotype')
    #Gpr88 = Phenotype('ilxtr:Gpr88', 'ilxtr:hasExpressionPhenotype')
    #Oxtr = Phenotype('ilxtr:Oxtr', 'ilxtr:hasExpressionPhenotype')
    #Tacr1 = Phenotype('ilxtr:Tacr1', 'ilxtr:hasExpressionPhenotype')
    #Hcrtr1 = Phenotype('ilxtr:Hcrtr1', 'ilxtr:hasExpressionPhenotype')
    #Opn3 = Phenotype('ilxtr:Opn3', 'ilxtr:hasExpressionPhenotype')
    # low unusual ??? check 4 H and enumerate all the instances of 'unusual'
    LPCDend = LogicalPhenotype(AND, Chrm2, Gpr88, Oxtr, Tacr1, Hcrtr1, Opn3)

    # encode the negative phenotypes 6 C, 6 has lots of useful information
    # specifically with regard to how to encode a thresholded negative phenotype

    # ilxtr:hasSynapticPhenotype
    #FastEPSP = Phenotype('ilxtr:FastEPSP', 'ilxtr:hasElectrophysiologicalPhenotype')

    # signaling
    RGS4 = Phenotype('ilxtr:RGS4', 'ilxtr:hasExpressionPhenotype')
    #Adcy8 = Phenotype('ilxtr:Adcy8', 'ilxtr:hasExpressionPhenotype')
    #Adcy1 = Phenotype('ilxtr:Adcy1', 'ilxtr:hasExpressionPhenotype')
    Ras111b = Phenotype('ilxtr:Ras111b', 'ilxtr:hasExpressionPhenotype')
    #Arhgef10 = Phenotype('ilxtr:Arhgef10', 'ilxtr:hasExpressionPhenotype')
    PVBCSig = LogicalPhenotype(AND, RGS4, Adcy8, Adcy1, Ras111b, Arhgef10)
    Gucy1a3 = Phenotype('ilxtr:Gucy1a3', 'ilxtr:hasExpressionPhenotype')
    Gucy1b3 = Phenotype('ilxtr:Gucy1b3', 'ilxtr:hasExpressionPhenotype')
    #Prkg1 = Phenotype('ilxtr:Prkg1', 'ilxtr:hasExpressionPhenotype')
    #Pde11a = Phenotype('ilxtr:Pde11a', 'ilxtr:hasExpressionPhenotype')
    #Pde5a = Phenotype('ilxtr:Pde5a', 'ilxtr:hasExpressionPhenotype')
    #Trpc5 = Phenotype('ilxtr:Trpc5', 'ilxtr:hasExpressionPhenotype')
    #Kcnmb2 = Phenotype('ilxtr:Kcnmb2', 'ilxtr:hasExpressionPhenotype')
    CHCSig = LogicalPhenotype(AND, Gucy1a3, Gucy1b3, Prkg1, Pde11a, Pde5a, Trpc5, Kcnmb2)
    #Rgs12 = Phenotype('ilxtr:Rgs12', 'ilxtr:hasExpressionPhenotype')
    #Adcy9 = Phenotype('ilxtr:Adcy9', 'ilxtr:hasExpressionPhenotype')
    #Pde7b = Phenotype('ilxtr:Pde7b', 'ilxtr:hasExpressionPhenotype')
    CCKCSig = LogicalPhenotype(AND, Rgs12, Adcy9, Pde7b)
    RGS6 = Phenotype('ilxtr:RGS6', 'ilxtr:hasExpressionPhenotype')
    RGS7 = Phenotype('ilxtr:RGS7', 'ilxtr:hasExpressionPhenotype')
    #Adcy2 = Phenotype('ilxtr:Adcy2', 'ilxtr:hasExpressionPhenotype')
    #Pde2a = Phenotype('ilxtr:Pde2a', 'ilxtr:hasExpressionPhenotype')
    MNCSig = LogicalPhenotype(AND, RGS6, RGS7, Adcy2, Pde2a)
    #Rgs16 = Phenotype('ilxtr:Rgs16', 'ilxtr:hasExpressionPhenotype')
    #Rgs10 = Phenotype('ilxtr:Rgs10', 'ilxtr:hasExpressionPhenotype')
    #Rgs8 = Phenotype('ilxtr:Rgs8', 'ilxtr:hasExpressionPhenotype')
    #Pde4b = Phenotype('ilxtr:Pde4b', 'ilxtr:hasExpressionPhenotype')
    ISCSig = LogicalPhenotype(AND, Rgs16, Rgs10, Rgs8, Pde4b)
    #Slc7a3 = Phenotype('ilxtr:Slc7a3', 'ilxtr:hasExpressionPhenotype')
    #Gucy1a3 = Phenotype('ilxtr:Gucy1a3', 'ilxtr:hasExpressionPhenotype')
    #Gucy1b3 = Phenotype('ilxtr:Gucy1b3', 'ilxtr:hasExpressionPhenotype')
    #Prkg2 = Phenotype('ilxtr:Prkg2', 'ilxtr:hasExpressionPhenotype')
    #Pde1a = Phenotype('ilxtr:Pde1a', 'ilxtr:hasExpressionPhenotype')
    #Trpc6 = Phenotype('ilxtr:Trpc6', 'ilxtr:hasExpressionPhenotype')
    #Kcnmb4 = Phenotype('ilxtr:Kcnmb4', 'ilxtr:hasExpressionPhenotype')
    LPCSig = LogicalPhenotype(AND, Slc7a3, Nos1, Gucy1a3, Gucy1b3, Prkg2, Pde1a, Trpc6, Kcnmb4)

    # axons
    Nav1_1 = Phenotype('ilxtr:Nav1_1', 'ilxtr:hasExpressionPhenotype')
    Nav1_6 = Phenotype('ilxtr:Nav1_6', 'ilxtr:hasExpressionPhenotype')
    Nav1_7 = Phenotype('ilxtr:Nav1_7', 'ilxtr:hasExpressionPhenotype')
    Cav2_1_P_Q = Phenotype('ilxtr:Cav2_1_P_Q', 'ilxtr:hasExpressionPhenotype')
    #Syt2 = Phenotype('ilxtr:Syt2', 'ilxtr:hasExpressionPhenotype')
    #Syt7 = Phenotype('ilxtr:Syt7', 'ilxtr:hasExpressionPhenotype')
    #Vamp1 = Phenotype('ilxtr:Vamp1', 'ilxtr:hasExpressionPhenotype')
    #Cplx1 = Phenotype('ilxtr:Cplx1', 'ilxtr:hasExpressionPhenotype')
    # FIXME listed as PV but probably means Pvalb?
    #high Snap25, Rab3a, NSF
    PVBCAxon = LogicalPhenotype(AND, Nav1_1, Nav1_6, Nav1_7, Cav2_1_P_Q, Pvalb, Syt2, Syt7, Vamp1, Cplx1)
    # lower Vamp1, Cplx1, Syt1, Syt2, Syt7 vs PVBC, probably want to encode the comparator expicitly
    # NOT generally
    # high Snap25, Rab3a, NSF
    TODO = Phenotype(ilxtr.to, ilxtr.hasPhenotype)
    CHCAxon = LogicalPhenotype(AND, TODO)
    #Cplx2 = Phenotype('ilxtr:Cplx2', 'ilxtr:hasExpressionPhenotype')
    #Cplx3 = Phenotype('ilxtr:Cplx3', 'ilxtr:hasExpressionPhenotype')
    LDCV = Phenotype('ilxtr:LDCV', 'ilxtr:hasExpressionPhenotype')
    #Syt10 = Phenotype('ilxtr:Syt10', 'ilxtr:hasExpressionPhenotype')
    CCKCAxon = LogicalPhenotype(AND, Cplx2, Cplx3, LDCV, Syt10)  # FIXME LDCV release?
    Znt3 = Phenotype('ilxtr:Znt3', 'ilxtr:hasExpressionPhenotype')
    Zip1 = Phenotype('ilxtr:Zip1', 'ilxtr:hasExpressionPhenotype')
    MNCAxon = LogicalPhenotype(AND, Znt3, Zip1)  # FIXME why does he have the names out front?
    ISCAxon = LogicalPhenotype(AND, *CCKCAxon.pes, Phenotype(ilxtr.similarTo, ilxtr.hasPhenotypeModifier))  # FIXME similar
    #Syt4 = Phenotype('ilxtr:Syt4', 'ilxtr:hasExpressionPhenotype')
    #Syt5 = Phenotype('ilxtr:Syt5', 'ilxtr:hasExpressionPhenotype')
    #Syt6 = Phenotype('ilxtr:Syt6', 'ilxtr:hasExpressionPhenotype')
    LPCAxon = LogicalPhenotype(AND, Syt4, Syt5, Syt6)

    # other
    PGC1α = Phenotype('ilxtr:PGC1α', 'ilxtr:hasExpressionPhenotype')
    #Esrrg = Phenotype('ilxtr:Esrrg', 'ilxtr:hasExpressionPhenotype')
    #Mef2c = Phenotype('ilxtr:Mef2c', 'ilxtr:hasExpressionPhenotype')
    #Pparg = Phenotype('ilxtr:Pparg', 'ilxtr:hasExpressionPhenotype')
    #Cox6c = Phenotype('ilxtr:Cox6c', 'ilxtr:hasExpressionPhenotype')
    #Nefh = Phenotype('ilxtr:Nefh', 'ilxtr:hasExpressionPhenotype')
    #Slit2 = Phenotype('ilxtr:Slit2', 'ilxtr:hasExpressionPhenotype')
    #Slit3 = Phenotype('ilxtr:Slit3', 'ilxtr:hasExpressionPhenotype')
    PVBCOther = LogicalPhenotype(AND, PGC1α, Esrrg, Mef2c, Pparg, Cox6c, Nefh, Slit2, Slit3)
    #Unc5b = Phenotype('ilxtr:Unc5b', 'ilxtr:hasExpressionPhenotype')
    #Hs3st4 = Phenotype('ilxtr:Hs3st4', 'ilxtr:hasExpressionPhenotype')
    CHCOther = LogicalPhenotype(AND, Unc5b, Hs3st4)
    #Hs6st3 = Phenotype('ilxtr:Hs6st3', 'ilxtr:hasExpressionPhenotype')
    CCKCOther = LogicalPhenotype(AND, Hs6st3)
    #Grin3a = Phenotype('ilxtr:Grin3a', 'ilxtr:hasExpressionPhenotype')
    MNCOther = LogicalPhenotype(AND, Grin3a)
    #Unc5a = Phenotype('ilxtr:Unc5a', 'ilxtr:hasExpressionPhenotype')
    #Chst1 = Phenotype('ilxtr:Chst1', 'ilxtr:hasExpressionPhenotype')
    ISCOther = LogicalPhenotype(AND, Unc5a, Chst1)
    #Ptn = Phenotype('ilxtr:Ptn', 'ilxtr:hasExpressionPhenotype')
    #Unc5d = Phenotype('ilxtr:Unc5d', 'ilxtr:hasExpressionPhenotype')
    LPCOther = LogicalPhenotype(AND, Ptn, Unc5d)


class Huang2017(Genes, Species):
    Neocortex = Phenotype('UBERON:0001950', 'ilxtr:hasSomaLocatedIn')
    Basket = Phenotype('ilxtr:BasketPhenotype', 'ilxtr:hasMorphologicalPhenotype')


with Huang2017:
    with Neuron_(Mouse, Neocortex) as context:
        #context.subClassOf(ilxtr.huang2017)
        # TODO add the names assigned here as abbrevs somehow

        #fig1b
        # FIXME is approach to disjointness VS adding negative phenotypes
        Neuron(Pvalb).disjointWith(Neuron(Sst), Neuron(Htr3a))
        #Neuron(Pvalb).equivalentClass(Neuron(NegPhenotype(Sst)), Neuron(NegPhenotype(Htr3a)))
        # FIXME for some reason the NegPhenotype neurons above fail to be added to the graph!??!
        #a = Neuron(NegPhenotype(Sst))
        #b = Neuron(NegPhenotype(Htr3a))
        #Neuron(Pvalb).equivalentClass(a, b)
        Neuron(Sst).disjointWith(Neuron(Htr3a))
        Neuron(Vip).subClassOf(Neuron(Htr3a))
        Neuron(Cck).subClassOf(Neuron(Htr3a))
        Neuron(Nkx2_1).disjointWith(Neuron(Nos1), Neuron(Htr3a), Neuron(Calb2))
        Neuron(Nos1).disjointWith(Neuron(Htr3a), Neuron(Calb2))

        Neuron(Vip, Cck)
        Neuron(Vip, Calb2, Cck)
        Neuron(Vip, Calb2)
        Neuron(Sst, Calb2)

        fig1a = dict(
        PVBC = Neuron(Basket, PV),
        CHC =  Neuron(Nkx2_1),
        CCKC = Neuron(Basket, VIP, CCK),
        MNC =  Neuron(SST, CR),
        ISC =  Neuron(VIP, CR),
        LPC =  Neuron(SST, NOS1),
        )

        f7 = dict(
        equivs =    (PVBCEq, CHCEq, CCKCEq, MNCEq, ISCEq, LPCEq),
        peptides =  (PVBCPep, CHCPep, CCKCPep, MNCPep, ISCPep, LPCPep),
        signaling = (PVBCSig, CHCSig, CCKCSig, MNCSig, ISCSig, LPCSig),
        dendrite =  (PVBCDend, CHCDend, CCKCDend, MNCDend, ISCDend, LPCDend),
        axon =      (PVBCAxon, CHCAxon, CCKCAxon, MNCAxon, ISCAxon, LPCAxon),
        other =     (PVBCOther, CHCOther, CCKCOther, MNCOther, ISCOther, LPCOther))

        figs7 = {type:Neuron(*(pe for p in phenos for pe in p.pes),
                            label=f'{type} all neuron', override=True)
                 # the zip below packs all PVBC with PVBC, all CHEC, etc.
                for type, *phenos in zip(fig1a, fig1a.values(), *f7.values())}

        for k, v in fig1a.items():
            # FIXME ISC currently classifies as vip cr cck which is incorrect
            # some _subset_ of those have cck, but it is not clear how many
            figs7[k].equivalentClass(v)  # TODO asserted by Josh Huang in figure s7

        peps = [Neuron(*p.pes, label=f'{l} peptides neuron', override=True)
                for l, p in zip(fig1a, f7['peptides'])]
        sigs = [Neuron(*p.pes, label=f'{l} signaling neuron', override=True)
                for l, p in zip(fig1a, f7['signaling'])]
        # asserted by Tom Gillespie interpreting Huang
        [p.equivalentClass(s) for p, s in zip(peps, sigs)]
        # assert that the peptide markers are disjoint
        for dis in (peps, sigs, tuple(fig1a.values())):
            for i, n in enumerate(dis[:-1]):
                for on in dis[i+1:]:
                    n.disjointWith(on)
        #LPCbyPepties = Neuron(*LPCPep.pes)

# common usage types
# allen 2016 hongwei

#embed()
for n, p in Huang2017.items():
    if isinstance(p, Phenotype) and not n.startswith('_'):
        # FIXME rdflib allows instances but tests type so OntId can go in, but won't ever match
        ident = OntId(p.p)
        if n in Genes.__dict__:
            o = rdflib.Literal(n) if not hasattr(p, '_label') else rdflib.Literal(p._label)
            lt = (rdflib.URIRef(ident), rdfs.label, o)
            Neuron.core_graph.add(lt)
            Neuron.out_graph.add(lt)  # FIXME maybe a helper graph?

            if ident.prefix == 'ilxtr' or ident.prefix == 'NCBIGene':  # FIXME NCBIGene temp fix ...
                if ident.suffix in ('LowerExpression', 'HigherExpression', 'to'):
                    continue
                sct = (rdflib.URIRef(ident), rdfs.subClassOf, ilxtr.gene)
                Neuron.core_graph.add(sct)
                Neuron.out_graph.add(sct)
        else:
            lt = (rdflib.URIRef(ident), rdfs.label, rdflib.Literal(OntTerm(ident).label))
            Neuron.core_graph.add(lt)
            Neuron.out_graph.add(lt)  # FIXME maybe a helper graph?

Neuron.out_graph.add((ilxtr.gene, owl.equivalentClass, OntId('SO:0000704').u))
Neuron.write()
Neuron.write_python()
res = [r for s, l in Neuron.out_graph[:rdfs.label:] if
       OntTerm(s).curie.startswith('ilxtr:')
       for r in OntTerm.query(label=l.toPython()) if
       r.curie.startswith('ilxtr:')]
mapped = [r.OntTerm for s, l in Neuron.out_graph[:rdfs.label:] if
          OntTerm(s).curie.startswith('ilxtr:')
          for r in OntTerm.query(label=l.toPython()) if
          not r.curie.startswith('ilxtr:')]

def ncbigenemapping():
    from pyontutils.config import devconfig
    from pathlib import Path
    import requests
    from bs4 import BeautifulSoup
    from lxml import etree
    asdf = {n:[qr.OntTerm.as_phenotype()
               for qr in OntTerm.query(term=n, prefix='NCBIGene')]
            for n, p in Genes.items()
            if not isinstance(p, LogicalPhenotype) and OntId(p.p).prefix != 'NCBIGene'}           
    may_need_ncbigene_added = [n for n, p in asdf.items() if not p]
    #urlbase = 'https://www.ncbi.nlm.nih.gov/gene/?term=Mus+musculus+'
    urlbase = ('https://www.ncbi.nlm.nih.gov/gene?term='
               '({gene_name}[Gene%20Name])%20AND%20{taxon_suffix}[Taxonomy%20ID]&'
               'report=xml')
    urls = [urlbase.format(gene_name=n, taxon_suffix=10090) for n in may_need_ncbigene_added]
    done2 = {}
    for u in urls:
        if u not in done2:
            print(u)
            done2[u] = requests.get(u)

    base = Path(devconfig.resources, 'genesearch')
    if not base.exists():
        base.mkdir()

    for resp in done2.values():
        fn = OntId(resp.url).quoted
        with open(base / fn, 'wb') as f:
            f.write(resp.content)

    so_much_soup = [BeautifulSoup(resp.content, 'lxml') for resp in done2.values()]

    trees = []
    for i, soup in enumerate(so_much_soup):
        pre = soup.find_all('pre')
        if pre:
            for p in pre[0].text.split('\n\n'):
                if p:
                    tree = etree.fromstring(p)
                    trees.append(tree)
        else:
            print('WAT', urls[i])

    dimension = 'ilxtr:hasExpressionPhenotype'
    errors = []
    to_add = []
    for tree in trees:
        taxon = tree.xpath('//Org-ref//Object-id_id/text()')[0]
        geneid = tree.xpath('//Gene-track_geneid/text()')[0]
        genename = tree.xpath('//Gene-ref_locus/text()')[0]
        if genename in may_need_ncbigene_added and taxon == '10090':
            print(f'{genename} = Phenotype(\'NCBIGene:{geneid}\', {dimension!r}, label={genename!r}, override=True)')
            to_add.append(geneid)
        else:
            errors.append((geneid, genename, taxon))

    print(errors)
    _ = [print('NCBIGene:' + ta) for ta in to_add]

    #wat.find_all('div', **{'class':'rprt-header'})
    #wat.find_all('div', **{'class':'ncbi-docsum'})

    replace = [print(n, '=', repr(p[0])) for n, p in asdf.items()
               if p and p[0].pLabel.toPython() == n]

    embed()


def main():
    embed()


if __name__ == '__main__':
    main()



import rdflib
from ontquery import OntCuries
from pyontutils import combinators as cmb
from pyontutils.core import OntId, OntTerm as OntTermBase
from pyontutils.namespaces import ilxtr
from pyontutils.closed_namespaces import rdf, rdfs, owl

a = rdf.type

current_collection = None
current_conventions = None


class Dimensions:
    molecular = ilxtr.hasMolecularPhenotype
    anatomical = ilxtr.hasLocationPhenotype
    ephys = ilxtr.hasElectrophysiologicalPhenotype

class CuriePrefixes:  # TODO populate these rationally
    molecular = (
        'CHEBI', 'PR', 'NCBIGene',
    )

    anatomical = (
        'UBERON',
        'FMA',
        'MBA',
        'PAXRAT',
    )

class Roots:
    molecular = (
        'ilxtr:gene', 'molecular entity', 'chemical entity'
    )


# init because some attrs will need to be switched
# to properties
dimensions = Dimensions()
curieprefixes = CuriePrefixes()
roots = Roots()


class OntTerm(OntTermBase):
    """ ask things about a term! """
    skip_for_instrumentation = True

    def isSubPropertyOf(self, property):
        return OntId(property) in self('rdfs:subPropertyOf',
                                       depth=20,
                                       direction='OUTGOING')

    def isSubClassOf(self, class_):
        return OntId(class_) in self('rdfs:subClassOf',
                                     depth=20,
                                     direction='OUTGOING')

    def _isType(self, name):
        raise NotImplementedError

    def isMolecular(self): return self._isType('molecular')
    def isAnatomical(self): return self._isType('anatomical')
    def isEphys(self): return self._isType('ephys')


class PredicateTerm(OntTermBase):
    skip_for_instrumentation = True
    __firsts = tuple()
    def _isType(self, name):
        return self.isSubPropertyOf(getattr(dimensions, name))

    def _isPropertyType(self, property):
        return self.isSubPropertyOf(property)


class ObjectTerm(OntTermBase):
    skip_for_instrumentation = True
    __firsts = tuple()
    def _isType(self, name):
        return (self.prefix in getattr(prefixes, name)
                or any(self.subClassOf(r) for r in getattr(roots, name)))

    def shortName(self, predicate):
        pass

    def longName(self, predicate):
        pass

    def localName(self, predicate):
        global current_conventions


class PhenotypeBase(tuple):
    _defaultPredicate = ilxtr.hasPhenotype
    def __new__(cls, value, dimension=_defaultPredicate):
        return super().__new__(cls, (ObjectTerm(value), PredicateTerm(dimension)))

    def __repr__(self):
        p = f', {self.predicate}' if self.predicate != self._defaultPredicate else ''
        return self.__class__.__name__ + f'({self.object}{p})'
    @property
    def predicate(self):
        # yes this is backwards, but we want it to match the args
        return self[1]

    @property
    def object(self):
        return self[0]

    def triples_individual(self, bnode_subject):
        yield bnode_subject, self.predicate.as_URIRef(), self.object.as_URIRef()

    def triples(self, bnode_subject):
        """ restriction version useful for debug """
        yield from self.combinator(bnode_subject)

    @property
    def combinator(self):
        """ yes, this is a property that returns a function """
        return cmb.restriction(self.predicate, self.object)

    def annotate(self, neuron, *annotations):
        """ in order to annotate a phenotype you must also
        supply a neuron in which it resides """

        neuron.annotate_phenotype(*annotations)

    def wrapName(self, name):
        return self.namePrefix + name + self.nameSuffix

    @property
    def shortName(self):
        return self.wrapName(self.object.shortName(self.predicate))

    @property
    def longName(self):
        return self.wrapName(self.object.longName(self.predicate))

    @property
    def localName(self):
        return self.wrapName(self.object.localName(self.predicate))


class Phenotype(PhenotypeBase):
    """ PositivePhenotype """


class NegativePhenotype(PhenotypeBase):
    pass


NegPhenotype = NegativePhenotype  # XXX legacy support


# phenotype collections


class PhenotypeCollection(frozenset):  # set? seems... fun? ordered set?
    """ untyped bags of atomic phenotypes """

    operator = None

    def __new__(cls, *phenotypes):
        self = super().__new__(cls, phenotypes)
        return self

    def __init__(self, *phenotypes):
        self.identifier = rdflib.BNode()
        pass

    @property
    def triples(self):
        list_ = cmb.List({owl.Restriction:cmb.Restriction(rdf.first)})
        yield self.identifier, a, owl.Class
        conbinators = (p.combinator for p in self)
        yield from list_.serialize(self.identifier, self.operator, *combinators)


PhenotypeCollection(Phenotype('ilxtr:someValue', 'ilxtr:someDimension'))


class LogicalPhenotype(PhenotypeCollection):
    """ useful for mapping naming conventions among other things """


# neurons

class NeuronBase(PhenotypeCollection):
    """ Immutable collections of phenotypes. """

    def annotate(self, *annotations):
        """ annotate the neuron as a whole """

    def annotate_phenotype(self, phenotype, *annotations):
        current_collection.annotation(self, phenotype, *annotations)


class AndNeuron(NeuronBase):
    operator = owl.intersectionOf


class OrNeuron(NeuronBase):
    operator = owl.unionOf


class Neuron(AndNeuron):
    pass


class QueryNeuron(OrNeuron):
    # FIXME not quite an OrNeuron because it is the intersection of
    # a QueryNeuron and the union of its phenotypes
    """ Parent class for all neurons that are used
        for bridging queries. """


class NeuronCollection:
    """ Heed the warnings of your ancestors!
        Just make another object. It will simplify your life!
        But do not do so needlessly! -- Ockham
    """

    def __init__(self,  # TODO the best parts from config ... (hah)
                 name=None,
                 comment=None,
                 metadata=None,):
        pass

    def activate(self):
        """ set this collection as the current
            active collection """
        global current_collection
        currrent_collection = self

    def add(self, neuron):
        # TODO, list? dict? set?
        pass

    # can't do from_python
    # this will be the config object
    # so the the latest neuron collection
    # will be where any new neurons are
    # automatically stored
    # one question is whether we use this with a context
    # manager or what ... why not both!

    def from_remote(self, url):
        """ could be a google sheet or a remote rdf representation """
    def from_file(self, filename, format=None): pass
    def from_graph(self, graph):
        """ triples can be a graph """

    def from_table(self, rows): pass

    def as_python(self): pass
    def as_owl(self, format='nifttl'):
        graph = rdflib.Graph()
        OntCuries.populate(graph)

    def as_table(self): pass

    def write_python(self, filename): pass
    def write_owl(self, filename): pass
    def write_table(self, filename): pass
    def upload_table(self, filename): pass



import re
from setuptools import setup


def find_version(filename):
    _version_re = re.compile(r"__version__ = '(.*)'")
    for line in open(filename):
        version_match = _version_re.match(line)
        if version_match:
            return version_match.group(1)


__version__ = find_version('neurondm/__init__.py')

with open('README.md', 'rt') as f:
    long_description = f.read()

tests_require = ['pytest', 'pytest-runner']
setup(
    name='neurondm',
    version=__version__,
    description='A data model for neuron types.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/tgbugs/pyontutils/tree/master/neurondm',
    author='Tom Gillespie',
    author_email='tgbugs@gmail.com',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    keywords=('neuron types NIF ontology neuroscience phenotype '
              'OWL rdf rdflib data model'),
    packages=['neurondm'],  # don't package models due to data resources needs?
    python_requires='>=3.6',
    tests_require=tests_require,
    install_requires=[
        'pyontutils>=0.1.2',
    ],
    extras_require={'dev': ['pytest-cov', 'wheel'],
                    'test': tests_require,
                    'notebook': ['jupyter'],
    },
    entry_points={
        'console_scripts': [
        ],
    },
)



import unittest
from neurondm import simple
from neurondm.simple import Phenotype, PhenotypeCollection


class TestSimple(unittest.TestCase):
    # TODO test_0_equality from test_neurons

    def test_phenotype(self):
        p1 = Phenotype('ilxtr:someValue', 'ilxtr:someDimension')
        p2 = Phenotype('ilxtr:someValue', 'ilxtr:someDimension')
        p3 = Phenotype('ilxtr:otherValue', 'ilxtr:someDimension')
        p4 = Phenotype('ilxtr:otherValue', 'ilxtr:otherDimension')
        assert p1 == p2 != p3 != p4

    def test_phenotype_collection(self):
        pc1 = PhenotypeCollection(Phenotype('ilxtr:someValue', 'ilxtr:someDimension'))
        pc2 = PhenotypeCollection(Phenotype('ilxtr:someValue', 'ilxtr:someDimension'))
        pc3 = PhenotypeCollection(Phenotype('ilxtr:otherValue', 'ilxtr:someDimension'))
        pc4 = PhenotypeCollection(Phenotype('ilxtr:otherValue', 'ilxtr:otherDimension'))
        assert pc1 == pc2 != pc3 != pc4

        pc = PhenotypeCollection(Phenotype('ilxtr:someValue', 'ilxtr:someDimension'),
                                 Phenotype('ilxtr:someValue', 'ilxtr:someDimension'))

        assert len(pc) == 1
        breakpoint()




import unittest
from pathlib import Path
import pytest
from pyontutils.utils import get_working_dir
from pyontutils.config import devconfig, checkout_ok
from pyontutils.integration_test_helper import _TestScriptsBase, Folders, Repo
import neurondm


class TestScripts(Folders, _TestScriptsBase):
    """ woo! """


only = tuple()

lasts = tuple()

neurons = ('neurondm/example',
           'neurondm/phenotype_namespaces',
           'neurondm/models/allen_cell_types',
           'neurondm/models/phenotype_direct',
           'neurondm/models/basic_neurons',
           'neurondm/models/huang2017',
           'neurondm/models/ma2015',
           'neurondm/models/cuts',
           'neurondm/build',
           'neurondm/sheets',)

skip = tuple()
if Path(devconfig.ontology_local_repo).exists():
    ont_repo = Repo(devconfig.ontology_local_repo)
    # FIXME these aren't called?
    post_load = lambda : (ont_repo.remove_diff_untracked(), ont_repo.checkout_diff_tracked())
    post_main = lambda : (ont_repo.remove_diff_untracked(), ont_repo.checkout_diff_tracked())

    ### handle ontology branch behavior
    print('checkout ok:', checkout_ok)
    ont_branch = ont_repo.active_branch.name
    if not checkout_ok and ont_branch != 'neurons':
        neurons += ('neurondm/core', 'neurondm/lang',)  # FIXME these two are ok for no repo but not wrong branch?!
        skip += tuple(n.split('/')[-1] for n in neurons)
    else:
        lasts += tuple(f'neurondm/{s}.py' for s in neurons)

else:
    skip += tuple(n.split('/')[-1] for n in neurons)


### build mains
mains = {}  # NOTE mains run even if this is empty ? is this desired?

module_parent = Path(__file__).resolve().parent.parent.as_posix()
working_dir = get_working_dir(__file__)
if working_dir is None:
    # python setup.py test will run from the module_parent folder
    # I'm pretty the split was only implemented because I was trying
    # to run all tests from the working_dir in one shot, but that has
    # a number of problems with references to local vs installed packages
    working_dir = module_parent

print(module_parent)
print(working_dir)

TestScripts.populate_tests(neurondm, working_dir, mains, skip=skip, lasts=lasts,
                           module_parent=module_parent, only=only, do_mains=True)
